# -*- coding: utf-8 -*-
# flake8: noqa

from collective.iconifiedcategory.utils import calculate_category_id
from collective.iconifiedcategory.utils import calculate_filesize
from collective.iconifiedcategory.utils import get_categorized_elements
from collective.iconifiedcategory.utils import get_categorized_infos
from collective.iconifiedcategory.utils import get_category_object
from collective.iconifiedcategory.utils import render_filesize
from collective.iconifiedcategory.utils import update_all_categorized_elements



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import testing
from zope.component import getUtility
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema.interfaces import IVocabularyFactory

import unittest


class TestVocabularies(unittest.TestCase):
    layer = testing.COLLECTIVE_ICONIFIED_CATEGORY_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def _check_category_vocabulary(self, vocabulary):
        """ """
        terms = [t.title for t in vocabulary]
        self.assertEqual(2 * 3 * 3, len(terms))
        categories = [
            'Category 1-3',
            'Category 2-3',
            'Category 1-2',
            'Category 2-2',
            'Category 1-1',
            'Category 2-1',
        ]
        self.assertListEqual(
            [t for t in terms if t.startswith('Category')],
            categories,
        )
        subcategories = [
            'Subcategory 1-3-2', 'Subcategory 1-3-1',
            'Subcategory 2-3-2', 'Subcategory 2-3-1',
            'Subcategory 1-2-2', 'Subcategory 1-2-1',
            'Subcategory 2-2-2', 'Subcategory 2-2-1',
            'Subcategory 1-1-2', 'Subcategory 1-1-1',
            'Subcategory 2-1-2', 'Subcategory 2-1-1',
        ]
        self.assertListEqual(
            [t for t in terms if t.startswith('Subcategory')],
            subcategories,
        )

    def test_category_vocabulary(self):
        vocabulary = getUtility(
            IVocabularyFactory,
            name='collective.iconifiedcategory.categories',
        )
        vocabulary = vocabulary(self.portal)
        self._check_category_vocabulary(vocabulary)

    def test_category_vocabulary_use_category_uid_as_token(self):
        vocabulary = getUtility(
            IVocabularyFactory,
            name='collective.iconifiedcategory.categories',
        )
        vocabulary = vocabulary(self.portal, use_category_uid_as_token=True)
        self._check_category_vocabulary(vocabulary)

    def test_category_title_vocabulary(self):
        vocabulary = getUtility(
            IVocabularyFactory,
            name='collective.iconifiedcategory.category_titles',
        )
        terms = [t.title for t in vocabulary(self.portal)]
        self.assertEqual(6, len(terms))
        expected = [
            'Category 1-1', 'Category 1-2', 'Category 1-3',
            'Category 2-1', 'Category 2-2', 'Category 2-3',
        ]
        self.assertListEqual(sorted(expected), sorted(terms))

        # add some predefined_title to some subcategories
        subcat = self.portal.config['group-1']['category-1-1']['subcategory-1-1-1']
        self.assertIsNone(subcat.predefined_title)
        subcat.predefined_title = u'Some predefined title'
        notify(ObjectModifiedEvent(subcat))
        terms = [t.title for t in vocabulary(self.portal)]
        self.assertEqual(7, len(terms))
        expected = [
            'Category 1-1', u'Some predefined title',
            'Category 1-2', 'Category 1-3',
            'Category 2-1', 'Category 2-2', 'Category 2-3',
        ]
        self.assertListEqual(sorted(expected), sorted(terms))

    def test_category_title_only_pdf_vocabulary(self):
        vocabulary = getUtility(
            IVocabularyFactory,
            name='collective.iconifiedcategory.categories',
        )
        # change only_pdf to True for a category and a subcategory
        cat = self.portal.config['group-1']['category-1-1']
        cat.only_pdf = True
        subcat = self.portal.config['group-1']['category-1-1']['subcategory-1-1-1']
        subcat.only_pdf = True

        terms = [t.title for t in vocabulary(self.portal)]
        self.assertEqual(
            terms,
            ['Category 1-3', 'Subcategory 1-3-2', 'Subcategory 1-3-1',
             'Category 2-3', 'Subcategory 2-3-2', 'Subcategory 2-3-1',
             'Category 1-2', 'Subcategory 1-2-2', 'Subcategory 1-2-1',
             'Category 2-2', 'Subcategory 2-2-2', 'Subcategory 2-2-1',
             'Category 1-1 [PDF!]', 'Subcategory 1-1-2', 'Subcategory 1-1-1 [PDF!]',
             'Category 2-1', 'Subcategory 2-1-2', 'Subcategory 2-1-1'])

    def test_category_title_show_preview_vocabulary(self):
        vocabulary = getUtility(
            IVocabularyFactory,
            name='collective.iconifiedcategory.categories',
        )
        # change only_pdf to True for a category and a subcategory
        cat = self.portal.config['group-1']['category-1-1']
        cat.show_preview = 1
        subcat = self.portal.config['group-1']['category-1-1']['subcategory-1-1-1']
        subcat.only_pdf = True
        subcat.show_preview = 2

        terms = [t.title for t in vocabulary(self.portal)]
        self.assertEqual(
            terms,
            ['Category 1-3', 'Subcategory 1-3-2', 'Subcategory 1-3-1',
             'Category 2-3', 'Subcategory 2-3-2', 'Subcategory 2-3-1',
             'Category 1-2', 'Subcategory 1-2-2', 'Subcategory 1-2-1',
             'Category 2-2', 'Subcategory 2-2-2', 'Subcategory 2-2-1',
             'Category 1-1 [Preview!]', 'Subcategory 1-1-2', 'Subcategory 1-1-1 [PDF!] [Preview!]',
             'Category 2-1', 'Subcategory 2-1-2', 'Subcategory 2-1-1'])



# -*- coding: utf-8 -*-

from collective.iconifiedcategory.tests.base import BaseTestCase


class TestContent(BaseTestCase):

    def test_edit_form(self):
        category_group = self.portal.config.get('group-1')
        category = category_group.get('category-1-1')
        subcategory = category.get('subcategory-1-1-1')

        # edit form is overrided for 'ContentCategory'
        self.assertEqual(category.portal_type, 'ContentCategory')
        edit = category.restrictedTraverse('@@edit')
        self.assertEqual(edit.context.get_category_group(), category_group)
        self.assertTrue(category.restrictedTraverse('@@edit')())

        # edit form is overrided for 'ContentSubcategory'
        self.assertEqual(subcategory.portal_type, 'ContentSubcategory')
        edit = subcategory.restrictedTraverse('@@edit')
        self.assertEqual(edit.context.get_category_group(), category_group)
        self.assertTrue(subcategory.restrictedTraverse('@@edit')())

    def test_content_view(self):
        category_group = self.portal.config.get('group-1')
        category = category_group.get('category-1-1')
        subcategory = category.get('subcategory-1-1-1')

        # just call the view for every contents
        self.assertTrue(category_group.restrictedTraverse('@@view')())
        self.assertTrue(category.restrictedTraverse('@@view')())
        self.assertTrue(subcategory.restrictedTraverse('@@view')())



# -*- coding: utf-8 -*-
from collective.iconifiedcategory.browser.views import CategorizedChildInfosView


class TestingCategorizedChildInfosView(CategorizedChildInfosView):

    def _show_protected_download(self, element):
        """ """
        return False



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _
from collective.iconifiedcategory.content.base import ICategorize
from collective.iconifiedcategory.interfaces import IICImageScaleTraversable
from plone.app.contenttypes.interfaces import IFolder
from plone.autoform import directives as form
from plone.dexterity.content import Container
from plone.dexterity.schema import DexteritySchemaPolicy
from plone.namedfile.field import NamedBlobImage
from plone.rfc822.interfaces import IPrimaryField
from zope.interface import alsoProvides
from zope.interface import implements


class ICategory(IFolder, ICategorize, IICImageScaleTraversable):

    form.order_before(icon='predefined_title')
    icon = NamedBlobImage(
        title=_(u'Icon'),
        description=_(u'Please use an image with size of 16x16, if different, '
                      u'the image will be reduced when displayed but this '
                      u'could be not optimal.'),
        required=True,
    )


alsoProvides(ICategory['icon'], IPrimaryField)


class Category(Container):
    implements(ICategory)

    @property
    def category_uid(self):
        return self.UID()

    @property
    def category_id(self):
        return self.getId()

    @property
    def category_title(self):
        return self.Title()

    def get_category_group(self, context=None):
        if context is None:
            context = self
        return context.aq_parent


class CategorySchemaPolicy(DexteritySchemaPolicy):

    def bases(self, schema_name, tree):
        return (ICategory, )



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory.interfaces import IIconifiedCategoryChangedEvent
from collective.iconifiedcategory.tests.base import BaseTestCase
from zope.component import adapter
from zope.component import getGlobalSiteManager


SUBSCRIBED_ELEMENTS = []


@adapter(IIconifiedCategoryChangedEvent)
def object_modified_subscriber(event):
    SUBSCRIBED_ELEMENTS.append(event.object.id)


class TestUpdateCategorizedElementsConfig(BaseTestCase):

    def setUp(self):
        super(TestUpdateCategorizedElementsConfig, self).setUp()
        SUBSCRIBED_ELEMENTS[:] = []
        self.gsm = getGlobalSiteManager()
        self.gsm.registerHandler(object_modified_subscriber)

    def tearDown(self):
        self.gsm.unregisterHandler(object_modified_subscriber)
        category = self.portal['config']['group-1']['category-1-1']
        category.title = 'Category 1-1'
        super(TestUpdateCategorizedElementsConfig, self).tearDown()

    def test_subscriber(self):
        """
        Test the ObjectModified event notifier for update-categorized-elements
        view on config context
        """
        config = self.portal['config']
        view = config.restrictedTraverse('@@update-categorized-elements')
        view()
        self.assertListEqual(['file_txt', 'image'], SUBSCRIBED_ELEMENTS)

    def test_result(self):
        """
        Test the update-categorized-elements view on config context
        """
        config = self.portal['config']
        category = config['group-1']['category-1-1']
        plone_file = self.portal['file_txt']
        plone_file.title = 'foo.txt'
        element = self.portal.categorized_elements[plone_file.UID()]
        self.assertEqual('Category 1-1', element['category_title'])
        category.title = 'Category 1-1 Modified'

        view = config.restrictedTraverse('@@update-categorized-elements')
        view()
        element = self.portal.categorized_elements[plone_file.UID()]
        self.assertEqual('Category 1-1 Modified', element['category_title'])
        # Title must be updated as well as limited=False
        self.assertEqual('foo.txt', element['title'])


class TestUpdateCategorizedElementsCategory(BaseTestCase):

    def setUp(self):
        super(TestUpdateCategorizedElementsCategory, self).setUp()
        SUBSCRIBED_ELEMENTS[:] = []
        self.gsm = getGlobalSiteManager()
        self.gsm.registerHandler(object_modified_subscriber)

    def tearDown(self):
        self.gsm.unregisterHandler(object_modified_subscriber)
        category = self.portal['config']['group-1']['category-1-1']
        category.title = 'Category 1-1'
        super(TestUpdateCategorizedElementsCategory, self).tearDown()

    def test_subscriber(self):
        """
        Test the ObjectModified event notifier for update-categorized-elements
        view on category context
        """
        config = self.portal['config']
        category = config['group-1']['category-1-1']['subcategory-1-1-1']
        view = category.restrictedTraverse('@@update-categorized-elements')
        view()
        self.assertListEqual(['image'], SUBSCRIBED_ELEMENTS)

    def test_result(self):
        """
        Test the update-categorized-elements view on category context
        """
        config = self.portal['config']
        category = config['group-1']['category-1-1']
        plone_file = self.portal['file_txt']
        element = self.portal.categorized_elements[plone_file.UID()]
        self.assertEqual('Category 1-1', element['category_title'])
        category.title = 'Category 1-1 Modified'

        view = category.restrictedTraverse('@@update-categorized-elements')
        view()
        element = self.portal.categorized_elements[plone_file.UID()]
        self.assertEqual('Category 1-1 Modified', element['category_title'])



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_base
from collections import OrderedDict
from collective.iconifiedcategory import _
from collective.iconifiedcategory import CAT_SEPARATOR
from collective.iconifiedcategory import CSS_SEPARATOR
from collective.iconifiedcategory import logger
from collective.iconifiedcategory.content.category import ICategory
from collective.iconifiedcategory.content.categorygroup import ICategoryGroup
from collective.iconifiedcategory.content.subcategory import ISubcategory
from collective.iconifiedcategory.interfaces import IIconifiedCategoryConfig
from collective.iconifiedcategory.interfaces import IIconifiedCategoryGroup
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from collective.iconifiedcategory.interfaces import IIconifiedContent
from collective.iconifiedcategory.interfaces import IIconifiedInfos
from datetime import datetime
from imio.helpers.cache import get_cachekey_volatile
from imio.helpers.content import find
from natsort import natsorted
from plone import api
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import IImage
from plone.memoize import ram
from Products.CMFPlone.utils import safe_unicode
from time import time
from zope.annotation import IAnnotations
from zope.component import getAdapter
from zope.component import getMultiAdapter
from zope.component import queryAdapter
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import Invalid

import copy


def format_id_css(id):
    return id.replace(CAT_SEPARATOR, CSS_SEPARATOR)


def query_config_root(context):
    """Try to get the categories config root for the given context"""
    adapter = queryAdapter(context, IIconifiedCategoryConfig)
    config_root = adapter and adapter.get_config() or None
    if not config_root and context is not None:
        catalog = api.portal.get_tool('portal_catalog')
        query = {
            'portal_type': 'ContentCategoryConfiguration',
        }
        result = catalog.unrestrictedSearchResults(**query)
        if not result:
            return
        config_root = result[0]._unrestrictedGetObject()
    return config_root


def has_config_root(context):
    """Verify if there is a config root for the given context"""
    return query_config_root(context) is not None


def get_config_root(context):
    """Return the categories config root for the given context"""
    config_root = query_config_root(context)
    if not config_root:
        raise ValueError('Categories config cannot be found')
    return get_group(config_root, context)


def get_group(config, context):
    """Return the associated groups for the given context"""
    adapter = getMultiAdapter((config, context), IIconifiedCategoryGroup)
    return adapter.get_group()


def get_categories(context,
                   the_objects=False,
                   only_enabled=True,
                   sort_on='getObjPositionInParent'):
    """Return the categories brains for a specific context"""
    config_root = get_config_root(context)
    config_group = get_group(config_root, context)
    catalog = api.portal.get_tool('portal_catalog')
    query = {
        'object_provides': 'collective.iconifiedcategory.content.category.ICategory',
    }
    # query on path is context is not the Plone Site
    # happens when computing categories to generate CSS
    if context.portal_type != "Plone Site":
        query['path'] = '/'.join(config_group.getPhysicalPath())
    if only_enabled:
        query['enabled'] = True
    if sort_on:
        query['sort_on'] = sort_on

    res = catalog.unrestrictedSearchResults(**query)
    if the_objects:
        res = [brain.getObject() for brain in res]
    return res


def calculate_category_id(category):
    """Return the caculated category id for a category object"""
    if ICategory.providedBy(category):
        return '{0}-{1}_-_{2}_-_{3}'.format(
            category.aq_parent.aq_parent.aq_parent.id,
            category.aq_parent.aq_parent.id,
            category.aq_parent.id,
            category.id,
        )
    if ISubcategory.providedBy(category):
        return '{0}-{1}_-_{2}_-_{3}_-_{4}'.format(
            category.aq_parent.aq_parent.aq_parent.aq_parent.id,
            category.aq_parent.aq_parent.aq_parent.id,
            category.aq_parent.aq_parent.id,
            category.aq_parent.id,
            category.id,
        )


def get_category_object(context, category_id):
    config_root = get_config_root(context)
    config_group = get_group(config_root, context)
    depth = 1
    if ICategoryGroup.providedBy(config_group):
        depth = 2
    category = config_group
    for path in category_id.split(CAT_SEPARATOR)[depth:]:
        category = category[path]
    if not ICategory.providedBy(category) and not ISubcategory.providedBy(category):
        raise KeyError
    return category


def get_category_icon_url(category):
    portal_url = api.portal.get_tool('portal_url')
    if ICategory.providedBy(category):
        obj = category
    else:
        obj = category.aq_parent

    # do not use restrictedTraverse or getMultiAdapter to get the "@@images" view
    # because when used with plone.app.async, as there is no REQUEST, it fails.
    from collective.iconifiedcategory.browser.views import ImageDataModifiedImageScaling
    images = ImageDataModifiedImageScaling(obj, getattr(obj, 'REQUEST', None))
    scale = images.scale(scale='listing')

    return u'{0}/@@images/{1}'.format(
        portal_url.getRelativeContentURL(obj),
        scale.__name__)


def update_categorized_elements(parent,
                                obj,
                                category,
                                limited=False,
                                sort=True,
                                logging=False):
    """ Update categorized elements
    parameters:
        - parent : The object parent
        - obj : The categorized element
        - category : The category object
        - limited : Update only category related informations
        - logging : Enables logging
    """
    if 'categorized_elements' not in parent.__dict__:
        parent.categorized_elements = OrderedDict()
    uid, new_infos = get_categorized_infos(obj, category, limited=limited)
    infos = parent.categorized_elements.get(uid, {})
    infos.update(new_infos)
    parent.categorized_elements[uid] = infos
    parent._p_changed = True
    if sort:
        sort_categorized_elements(parent)
    if logging:
        logger.info('Updated categorized elements of {0}'.format(
            obj.absolute_url_path()))


def update_all_categorized_elements(container, limited=False, sort=True):
    # recompute everything if limited=False
    if not limited:
        container.categorized_elements = OrderedDict()
    adapter = None
    for obj in container.objectValues():
        if hasattr(obj, 'content_category'):
            try:
                category = get_category_object(obj, obj.content_category)
            except KeyError:
                continue
            if adapter is None:
                adapter = getAdapter(obj, IIconifiedInfos)
            else:
                adapter.context = obj
                adapter.obj = aq_base(obj)
            uid, new_infos = obj.UID(), adapter.get_infos(category, limited=limited)
            infos = container.categorized_elements.get(uid, {})
            infos.update(new_infos)
            container.categorized_elements[uid] = infos
    if container.categorized_elements and sort:
        sort_categorized_elements(container)


def get_ordered_categories_cachekey(method, context, only_enabled=True):
    """Makes cache depend on config root."""
    return (repr(get_config_root(context)),
            get_cachekey_volatile("collective.iconifiedcategory.utils.get_ordered_categories"))


@ram.cache(get_ordered_categories_cachekey)
def get_ordered_categories(context, only_enabled=True):
    """Return an ordered dict for categories (id and uids)"""
    elements = {}
    config_root = get_config_root(context)
    adapter = getMultiAdapter((config_root, context), IIconifiedCategoryGroup)
    categories = adapter.get_every_categories(only_enabled=only_enabled)
    query = {}
    query['object_provides'] = 'collective.iconifiedcategory.content.subcategory.ISubcategory'
    if only_enabled:
        query['enabled'] = True
    for idx, category in enumerate(categories):
        elements[category.UID()] = idx
        elements[calculate_category_id(category)] = idx
        subcategories = find(context=category, unrestricted=True, **query)
        for subcategory in subcategories:
            elements[subcategory.UID] = idx
            elements[calculate_category_id(subcategory.getObject())] = idx
    return elements


def sort_categorized_elements(context):
    """Sort the categorized elements on an object"""
    ordered_categories = get_ordered_categories(context, only_enabled=False)
    # use realsorted on a lowered title so it mixes uppercase and lowercase titles
    try:
        elements = natsorted(
            context.categorized_elements.items(),
            key=lambda x: (ordered_categories[x[1]['category_uid']],
                           safe_unicode(x[1]['title'].lower()),),
        )
    except KeyError:
        return
    context.categorized_elements = OrderedDict([(k, v) for k, v in elements])
    context._p_changed = True


def remove_categorized_element(parent, obj):
    if obj.UID() in getattr(parent, 'categorized_elements', OrderedDict()):
        del parent.categorized_elements[obj.UID()]


def get_categorized_infos(obj, category, limited=False):
    adapter = getAdapter(obj, IIconifiedInfos)
    return obj.UID(), adapter.get_infos(category, limited=limited)


def _categorized_elements(context):
    """Return a deepcopy of the categorized elements of the given context"""
    return copy.deepcopy(
        getattr(aq_base(context), 'categorized_elements', OrderedDict())
    )


def _check_filters(infos, filters):
    """ """
    keep = True
    for k, v in filters.items():
        # manage case when stored value is a list or not
        stored_value = infos[k]
        if not hasattr(stored_value, '__iter__'):
            stored_value = (stored_value, )
        if v not in stored_value:
            keep = False
            break
    return keep


def _get_adapter(context, obj, adapter):
    """ """
    if adapter is None:
        adapter = getMultiAdapter(
            (context, context.REQUEST, obj),
            IIconifiedContent)
    else:
        adapter.categorized_obj = obj
    return adapter


def get_categorized_elements(context,
                             result_type='dict',
                             portal_type=None,
                             sort_on=None,
                             uids=[],
                             filters={},
                             check_can_view=True,
                             caching=True):
    """Return categorized elements.
       p_result_type may be :
       - 'dict': default, essential metadata are returned as a dict;
       - 'objects': categorized objects are returned.
       If some p_filters are given, the values will be filtered,
       available filters are values stored in categorized_elements.
       If p_check_can_view is True, then the IIconifiedContent.can_view
       check will be called."""
    elements = None
    if caching:
        # in some cases like in tests, request can not be retrieved
        key = "collective.iconifiedcategory.get_categorized_elements'" \
            "-{0}-{1}-{2}-{3}-{4}".format(
                repr(context),
                result_type,
                portal_type,
                '_'.join(uids),
                ['{0}_{1}'.format(k, v) for k, v in filters.items()])
        cache = IAnnotations(context.REQUEST)
        elements = cache.get(key, None)

    if elements is None:
        elements = []
        categorized_elements = _categorized_elements(context)
        if not categorized_elements:
            return elements

        uids = uids or categorized_elements.keys()
        catalog = api.portal.get_tool('portal_catalog')
        current_user_allowedRolesAndUsers = catalog._listAllowedRolesAndUsers(api.user.get_current())
        adapter = None
        for uid, infos in categorized_elements.items():
            if (uids and uid not in uids) or \
               (portal_type and infos['portal_type'] != portal_type) or \
               not _check_filters(infos, filters) or \
               (infos['confidential'] and not set(infos['allowedRolesAndUsers'])
                   .intersection(current_user_allowedRolesAndUsers)):
                continue

            obj = context.get(infos['id'])
            adapter = _get_adapter(context, obj, adapter)
            if not check_can_view or adapter.can_view():
                if result_type == 'objects':
                    elements.append(obj)
                else:
                    # add 'UID' to the available infos
                    tmp = categorized_elements[uid].copy()
                    tmp['UID'] = uid
                    elements.append(tmp)

        if elements and sort_on:
            if result_type == 'dict':
                if sort_on in elements[0]:
                    elements = sorted(elements, key=lambda x, sort_on=sort_on: x[sort_on])
                elif sort_on == 'getObjPositionInParent':
                    elements = sorted(
                        elements,
                        key=lambda x, object_ids=context.objectIds(): object_ids.index(x['id']))
            else:
                if getattr(elements[0], sort_on):
                    elements = sorted(elements, key=lambda x, sort_on=sort_on: getattr(x, sort_on))
                elif sort_on == 'getObjPositionInParent':
                    elements = sorted(
                        elements,
                        key=lambda x, object_ids=context.objectIds(): object_ids.index(x.id))

    return elements


def get_back_references(obj):
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog.unrestrictedSearchResults(content_category_uid=obj.UID())
    return brains


def has_relations(obj):
    for relation in get_back_references(obj):
        return True
    if ICategory.providedBy(obj):
        for subcategory in obj.listFolderContents():
            for relation in get_back_references(subcategory):
                return True
    return False


def calculate_filesize(size):
    unit = 'B'
    factor = 1
    sizes = {
        1024. * 1024 * 1024 * 1024: 'TB',
        1024. * 1024 * 1024: 'GB',
        1024. * 1024: 'MB',
        1024.: 'KB',
    }
    for s, u in sizes.items():
        if size >= s:
            unit = u
            factor = s
            break
    size = round(size / factor, 1)
    if unit in ('B', 'KB'):
        size = int(size)
    return '{0} {1}'.format(size, unit)


def warn_filesize(size):
    filesizelimit = api.portal.get_registry_record(
        'filesizelimit',
        interface=IIconifiedCategorySettings,
    )
    if size > filesizelimit:
        return True
    return False


def render_filesize(size):
    pretty_filesize = calculate_filesize(size)
    if warn_filesize(size):
        pretty_filesize = \
            u"<span class='warn_filesize' title='{0}'>{1}</span>".format(
                translate('help_warn_filesize',
                          domain='collective.iconifiedcategory',
                          context=getRequest(),
                          default='Annex size is huge, it could '
                          'be difficult to be downloaded!'),
                pretty_filesize)
    elif not pretty_filesize.endswith((' B', ' KB')):
        pretty_filesize = \
            u"<span class='soft_warn_filesize'>{0}</span>".format(pretty_filesize)
    return pretty_filesize


def print_message(obj=None, to_print_value=None):
    """Return the print status message for the given object"""
    messages = {
        True: u'Must be printed',
        False: u'Should not be printed',
        None: u'Not convertible to a printable format',
    }
    if obj:
        return messages.get(obj.to_print, getattr(obj, 'to_print_message', ''))
    else:
        return messages[to_print_value]


def signed_message(obj=None, to_sign_value=None, signed_value=None):
    """Return the signed message for the given object"""
    messages = {
        False: u'Element must be signed but is still not',
        True: u'Element is signed',
    }
    not_to_sign_msg = u'Element should not be signed'
    if obj:
        if getattr(obj, 'to_sign', False) is False:
            return not_to_sign_msg
        return messages.get(getattr(obj, 'signed', False), '')
    elif to_sign_value is False:
        return not_to_sign_msg
    else:
        return messages[signed_value]


def boolean_message(obj=None, attr_name='', value=None):
    """Return the default boolean status message for the given object"""
    messages = {
        True: u'Element is {0}'.format(attr_name),
        False: u'Element is not {0}'.format(attr_name),
    }
    if obj:
        return messages.get(getattr(obj, attr_name, None), '')
    else:
        return messages[value]


@ram.cache(lambda f, p: (p, time() // (60 * 60)))
def is_file_type(portal_type):
    """Verify if the given portal type provides IFile or IImage"""
    portal_type = api.portal.get_tool('portal_types')[portal_type]
    module_path, classname = (
        u'.'.join(portal_type.klass.split('.')[:-1]),
        portal_type.klass.split('.')[-1],
    )
    module = __import__(module_path, {}, {}, [classname])
    cls = getattr(module, classname, None)
    if cls is None:
        return False
    for interface in (IFile, IImage):
        if cls.__implemented__(interface):
            return True
    return False


def validateFileIsPDF(data):
    """May be used as helper in a invariant validator"""
    def _get_content_type(data):
        """ """
        contentType = getattr(data, 'contentType', None)
        if contentType is None:
            file = request.form.get('form.widgets.file') or \
                getattr(aq_base(context), 'file', None)
            # get contentType
            if file:
                contentType = getattr(file, 'contentType', None) or file.headers.get('content-type')
        return contentType

    # check if file contentType is PDF only if used content_category requires it
    request = getRequest()
    # avoid double validation
    if request.get('already_validateFileIsPDF', False):
        return
    request.set('already_validateFileIsPDF', True)
    context = data.__context__ or request.get('PUBLISHED').context
    contentType = _get_content_type(data)
    if contentType is not None and contentType != 'application/pdf':
        category = get_category_object(context, data.content_category)
        if category.only_pdf:
            raise Invalid(_(u"You must select a PDF file!"))


def _modified(obj, asdatetime=True):
    """Returns max value between obj.modified() and obj._p_mtime,
       in case an annotation is changed on obj, obj._p_mtime is changed,
       not obj.modified()."""
    modified = max(float(obj.modified()), obj._p_mtime)
    if asdatetime:
        modified = datetime.fromtimestamp(modified)
    return modified



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collections import OrderedDict
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from collective.iconifiedcategory.tests.base import BaseTestCase
from plone import api
from plone.dexterity.utils import createContentInContainer
from zExceptions import Redirect

import transaction


class TestUtils(BaseTestCase):

    def setUp(self):
        super(TestUtils, self).setUp()
        elements = ('file_txt', 'image')
        for element in elements:
            if element in self.portal:
                api.content.delete(self.portal[element])

    def test_category_before_remove(self):
        """
        Ensure that an error is raised if we try to remove an used category
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        document = api.content.create(
            type='Document',
            title='doc-category-remove',
            container=self.portal,
            content_category='config_-_group-1_-_category-x',
            to_print=False,
            confidential=False,
        )
        self.assertRaises(Redirect, api.content.delete, category)
        api.content.delete(document)
        api.content.delete(category)

    def test_category_before_remove_while_removing_plone_site(self):
        """
        Removing the Plone Site is not prohibited if categories exist.
        """
        app = self.portal.aq_inner.aq_parent
        self.assertEqual(self.portal.getId(), 'plone')
        self.assertTrue('plone' in app.objectIds())
        app.manage_delObjects(ids=['plone'])
        self.assertFalse('plone' in app.objectIds())

    def test_category_with_subcategory_before_remove(self):
        """
        Ensure that an error is raised if we try to remove a category that
        contains an used subcategory
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        api.content.create(
            type='ContentSubcategory',
            title='Subcategory X',
            icon=self.icon,
            container=category,
        )
        document = api.content.create(
            type='Document',
            title='doc-category-remove-2',
            container=self.portal,
            content_category='config_-_group-1_-_category-x_-_subcategory-x',
            to_print=False,
            confidential=False,
        )
        self.assertRaises(Redirect, api.content.delete, category)
        api.content.delete(document)
        api.content.delete(category)

    def test_subcategory_before_removed(self):
        """
        Ensure that an error is raised if we try to remove an used subcategory
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        subcategory = api.content.create(
            type='ContentSubcategory',
            title='Subcategory X',
            icon=self.icon,
            container=category,
        )
        document = api.content.create(
            type='Document',
            title='doc-subcategory-remove',
            container=self.portal,
            content_category='config_-_group-1_-_category-x_-_subcategory-x',
            to_print=False,
            confidential=False,
        )
        self.assertRaises(Redirect, api.content.delete, subcategory)
        api.content.delete(document)
        api.content.delete(subcategory)
        api.content.delete(category)

    def test_category_moved(self):
        """
        Ensure that an error is raised if we try to move an used category
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        document = api.content.create(
            type='Document',
            title='doc-category-move-1',
            container=self.portal,
            content_category='config_-_group-1_-_category-x',
            to_print=False,
            confidential=False,
        )
        self.assertRaises(Redirect, api.content.move, category,
                          self.config['group-2'])
        api.content.delete(document)
        category = api.content.move(category, self.config['group-2'])
        api.content.delete(category)

    def test_category_subcategory_moved(self):
        """
        Ensure that an error is raised if we try to move a category that
        contains an used subcategory
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        api.content.create(
            type='ContentSubcategory',
            title='Subcategory X',
            icon=self.icon,
            container=category,
        )
        document = api.content.create(
            type='Document',
            title='doc-category-move-2',
            container=self.portal,
            content_category='config_-_group-1_-_category-x_-_subcategory-x',
            to_print=False,
            confidential=False,
        )
        new_folder = self.config['group-1']
        self.assertRaises(Redirect, api.content.move, category, new_folder)
        api.content.delete(document)
        category = api.content.move(category, new_folder)
        api.content.delete(category)

    def test_subcategory_moved(self):
        """
        Ensure that an error is raised if we try to move an used subcategory
        """
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        subcategory = api.content.create(
            type='ContentSubcategory',
            title='Subcategory X',
            icon=self.icon,
            container=category,
        )
        document = api.content.create(
            type='Document',
            title='doc-subcategory-move',
            container=self.portal,
            content_category='config_-_group-1_-_category-x_-_subcategory-x',
            to_print=False,
            confidential=False,
        )
        new_folder = self.config['group-1']['category-1-1']
        self.assertRaises(Redirect, api.content.move, subcategory, new_folder)
        api.content.delete(document)
        subcategory = api.content.move(subcategory, new_folder)
        api.content.delete(subcategory)
        api.content.delete(category)

    def test_calculate_filesize(self):
        self.assertEqual('100 B', utils.calculate_filesize(100))
        self.assertEqual('1 KB', utils.calculate_filesize(1024))
        self.assertEqual('1.1 MB', utils.calculate_filesize(1150976))
        self.assertEqual('15.5 MB', utils.calculate_filesize(16252928))

    def test_print_message(self):
        obj = type('obj', (object, ), {
            'to_print': False,
        })()
        self.assertEqual(u'Should not be printed', utils.print_message(obj))

        obj.to_print = True
        self.assertEqual(u'Must be printed', utils.print_message(obj))

        obj.to_print = None
        self.assertEqual(u'Not convertible to a printable format',
                         utils.print_message(obj))

    def test_boolean_message(self):
        obj = type('obj', (object, ), OrderedDict())()
        self.assertEqual(u'', utils.boolean_message(obj))

        obj.confidential = True
        self.assertEqual(
            u'Element is confidential',
            utils.boolean_message(obj, attr_name='confidential'))
        obj.publishable = True
        self.assertEqual(
            u'Element is publishable',
            utils.boolean_message(obj, attr_name='publishable'))

        obj.confidential = False
        self.assertEqual(
            u'Element is not confidential',
            utils.boolean_message(obj, attr_name='confidential'))
        obj.publishable = False
        self.assertEqual(
            u'Element is not publishable',
            utils.boolean_message(obj, attr_name='publishable'))

    def test_warn_filesize(self):
        # default warning is for files > 5Mb
        self.assertEqual(
            api.portal.get_registry_record(
                'filesizelimit',
                interface=IIconifiedCategorySettings,
            ),
            5000000)
        file1 = api.content.create(
            id='file1',
            type='File',
            file=self.file,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        self.assertEqual(file1.file.size, 3017)
        self.assertFalse(utils.warn_filesize(file1.file.size))

        # now enable warning (a specific portal_message is added when file created)
        api.portal.set_registry_record(
            'filesizelimit',
            interface=IIconifiedCategorySettings,
            value=3000)
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        self.assertEqual(file2.file.size, 3017)
        self.assertTrue(utils.warn_filesize(file2.file.size))

    def test_render_filesize(self):
        self.assertEqual(utils.render_filesize(1000),
                         '1000 B')
        self.assertEqual(utils.render_filesize(1024),
                         '1 KB')
        self.assertEqual(utils.render_filesize(5000),
                         '4 KB')
        # soft warning if filesize no more in B/KB
        self.assertEqual(utils.render_filesize(2500000),
                         u"<span class='soft_warn_filesize'>2.4 MB</span>")
        self.assertEqual(utils.render_filesize(5000000),
                         u"<span class='soft_warn_filesize'>4.8 MB</span>")
        # warning if filesize > 5000000
        self.assertEqual(utils.render_filesize(5000001),
                         u"<span class='warn_filesize' title='Annex size is huge, "
                         "it could be difficult to be downloaded!'>4.8 MB</span>")

    def test_get_categorized_elements(self):
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        transaction.commit()
        document = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='doc-subcategory-move',
            description='Document description',
            content_category='config_-_group-1_-_category-x',
            to_print=False,
            confidential=False,
        )
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        # while creating, modified is changed again after categorized_update
        utils.update_all_categorized_elements(self.portal)
        result = utils.get_categorized_elements(self.portal)
        self.assertEqual(
            result,
            [{'UID': document.UID(),
              'allowedRolesAndUsers': ['Anonymous'],
              'category_id': 'category-x',
              'category_title': 'Category X',
              'category_uid': category.UID(),
              'confidential': False,
              'confidentiality_activated': False,
              'contentType': None,
              'description': 'Document description',
              'download_url': None,
              'filesize': None,
              'icon_url': u'config/group-1/category-x/@@images/{0}'.format(scale),
              'id': 'doc-subcategory-move',
              'last_updated': self._modified(document),
              'portal_type': 'Document',
              'preview_status': 'not_convertable',
              'publishable': False,
              'publishable_activated': False,
              'relative_url': 'doc-subcategory-move',
              'show_preview': 0,
              'signed': False,
              'signed_activated': False,
              'subcategory_id': None,
              'subcategory_title': None,
              'subcategory_uid': None,
              'title': 'doc-subcategory-move',
              'to_be_printed_activated': True,
              'to_print': False,
              'to_sign': False,
              'warn_filesize': False}])
        # filter on portal_type
        self.assertEqual(
            utils.get_categorized_elements(self.portal,
                                           portal_type='Document'),
            result)
        self.failIf(utils.get_categorized_elements(self.portal,
                                                   portal_type='Document2'))
        # ask the objects
        self.assertEqual(
            utils.get_categorized_elements(self.portal,
                                           result_type='objects'),
            [document])

        # sort_on
        document2 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='2doc-subcategory-move',
            content_category='config_-_group-1_-_category-x',
            to_print=False,
            confidential=False,
        )
        # result_type='dict'
        result = utils.get_categorized_elements(self.portal, sort_on='title')
        expected = [res['title'] for res in result]
        self.assertEqual(expected, ['2doc-subcategory-move', 'doc-subcategory-move'])

        # result_type='objects'
        self.assertEqual(
            set(utils.get_categorized_elements(self.portal,
                                               result_type='objects')),
            set([document, document2]))
        self.assertEqual(
            utils.get_categorized_elements(self.portal,
                                           result_type='objects',
                                           sort_on='title'),
            [document2, document])
        # sort_on='getObjPositionInParent'
        self.assertEqual(
            [elt['UID'] for elt in
             utils.get_categorized_elements(self.portal,
                                            result_type='dict',
                                            sort_on='getObjPositionInParent')],
            [document.UID(), document2.UID()])
        # change document position
        self.portal.folder_position(position='up', id=document2.getId())
        # sort_on='getObjPositionInParent'
        self.assertEqual(
            [elt['UID'] for elt in
             utils.get_categorized_elements(self.portal,
                                            result_type='dict',
                                            sort_on='getObjPositionInParent')],
            [document2.UID(), document.UID()])

        # teardown
        self.assertRaises(Redirect, api.content.delete, category)
        api.content.delete(document)
        api.content.delete(document2)
        api.content.delete(category)

    def test_get_categorized_elements_filters(self):
        self.config.get('group-1').confidentiality_activated = True
        createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='Doc1',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=True,
        )
        self.assertEqual(len(utils.get_categorized_elements(
            self.portal, filters={'confidential': True})),
            1)
        self.assertEqual(len(utils.get_categorized_elements(
            self.portal, filters={'confidential': False})),
            0)

    def test_update_categorized_elements(self):
        document2 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='CV Info N\xc2\xb02016-2',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        document3 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='CV Info N\xc2\xb02016-1',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        document10 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='doc10',
            content_category='config_-_group-1_-_category-1-2',
            to_print=False,
            confidential=False,
        )
        document1 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='doc1',
            content_category='config_-_group-1_-_category-1-2',
            to_print=False,
            confidential=False,
        )
        document4 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='Doc4',
            content_category='config_-_group-1_-_category-1-2',
            to_print=False,
            confidential=False,
        )

        self.assertEqual(
            [cat.id for cat in utils.get_categories(document1, the_objects=True)],
            ['category-1-3', 'category-2-3', 'category-1-2',
             'category-2-2', 'category-1-1', 'category-2-1']
        )

        # order is respected, by category
        result = ['doc1', 'Doc4', 'doc10', 'CV Info N\xc2\xb02016-1', 'CV Info N\xc2\xb02016-2']
        self.assertEqual(
            result,
            [e['title'] for e in self.portal.categorized_elements.values()],
        )
        api.content.delete(document1)
        api.content.delete(document2)
        api.content.delete(document3)
        api.content.delete(document10)
        api.content.delete(document4)

    def test_update_all_categorized_elements(self):
        document1 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='doc1',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        document1UID = document1.UID()
        document2 = createContentInContainer(
            container=self.portal,
            portal_type='Document',
            title='doc2',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        document2UID = document2.UID()
        self.assertEqual(len(self.portal.categorized_elements), 2)
        self.assertTrue(document1UID in self.portal.categorized_elements)
        self.assertTrue(document2UID in self.portal.categorized_elements)
        self.portal.categorized_elements = OrderedDict()
        self.assertEqual(len(self.portal.categorized_elements), 0)
        utils.update_all_categorized_elements(self.portal)
        self.assertEqual(len(self.portal.categorized_elements), 2)
        self.assertTrue(document1UID in self.portal.categorized_elements)
        self.assertTrue(document2UID in self.portal.categorized_elements)

        # if a content_category is wrong, element is no more stored in categorized_elements
        document1.content_category = 'some_wrong_category_id'
        utils.update_all_categorized_elements(self.portal)
        self.assertEqual(len(self.portal.categorized_elements), 1)
        self.assertTrue(document2UID in self.portal.categorized_elements)

    def test_get_category_icon_url(self):
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        subcategory = api.content.create(
            type='ContentSubcategory',
            title='Subcategory X',
            container=category,
        )
        transaction.commit()
        document = api.content.create(
            type='Document',
            title='doc-subcategory-remove',
            container=self.portal,
            content_category='config_-_group-1_-_category-x_-_subcategory-x',
            to_print=False,
            confidential=False,
        )

        subcategory = utils.get_category_object(document, document.content_category)
        doc_icon_url = utils.get_category_icon_url(subcategory)
        category = subcategory.get_category()
        scale = category.restrictedTraverse('@@images').scale(scale='listing').__name__
        self.assertEqual(doc_icon_url, u'config/group-1/category-x/@@images/{0}'.format(scale))

    def test_get_ordered_categories(self):
        """Test that caching is behaving correctly."""
        # invalidated when new category added
        category = api.content.create(
            type='ContentCategory',
            title='Category X',
            icon=self.icon,
            container=self.config['group-1'],
        )
        category_uid = category.UID()
        self.assertTrue(category_uid in utils.get_ordered_categories(self.portal))
        # invalidated when category deleted
        api.content.delete(category)
        self.assertFalse(category_uid in utils.get_ordered_categories(self.portal))
        # invalidated when a category position changed
        res = sorted(utils.get_ordered_categories(self.portal).items())
        self.portal.config.get('group-1').moveObjectsDown('category-1-2')
        res_after_cat_position_changed = sorted(utils.get_ordered_categories(self.portal).items())
        self.assertNotEqual(res, res_after_cat_position_changed)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import testing
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.event import IconifiedAttrChangedEvent
from collective.iconifiedcategory.tests.base import BaseTestCase
from plone import api
from Products.CMFPlone.utils import base_hasattr

import unittest


class TestIconifiedChangedEvent(unittest.TestCase):
    layer = testing.COLLECTIVE_ICONIFIED_CATEGORY_FUNCTIONAL_TESTING

    def setUp(self):
        from zope.event import subscribers
        self._old_subscribers = subscribers[:]
        subscribers[:] = []

    def tearDown(self):
        from zope.event import subscribers
        subscribers[:] = self._old_subscribers

    def _notify(self, event):
        from zope.event import notify
        notify(event)

    def test_iconifiedattrchangedevent(self):
        from zope.event import subscribers
        dummy = []
        subscribers.append(dummy.append)
        event = IconifiedAttrChangedEvent(object(), '', 'old', 'new')
        self._notify(event)
        self.assertEqual(dummy, [event])


class TestTriggeredEvents(BaseTestCase, unittest.TestCase):

    def test_categorized_elements_correct_after_copy_paste_categorized_content(self):
        file_obj = self.portal['file_txt']
        file_obj_UID = file_obj.UID()
        img_obj = self.portal['image']
        img_obj_UID = img_obj.UID()
        self.assertEquals(len(self.portal.categorized_elements), 2)
        self.assertTrue(file_obj_UID in self.portal.categorized_elements)
        self.assertTrue(img_obj_UID in self.portal.categorized_elements)

        # copy paste a contained categorized content
        copied_data = self.portal.manage_copyObjects(ids=[file_obj.getId()])
        infos = self.portal.manage_pasteObjects(copied_data)
        new_file = self.portal[infos[0]['new_id']]
        new_file_UID = new_file.UID()
        self.assertEquals(len(self.portal.categorized_elements), 3)
        self.assertTrue(file_obj_UID in self.portal.categorized_elements)
        self.assertTrue(img_obj_UID in self.portal.categorized_elements)
        self.assertTrue(new_file_UID in self.portal.categorized_elements)

    def test_categorized_elements_correct_after_copy_paste_categorized_content_container(self):
        container = api.content.create(
            id='folder',
            type='Folder',
            container=self.portal
        )
        file_obj1 = api.content.create(
            id='file1',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        file_obj1_UID = file_obj1.UID()
        file_obj2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        file_obj2_UID = file_obj2.UID()
        self.assertEquals(len(container.categorized_elements), 2)
        self.assertTrue(file_obj1_UID in container.categorized_elements)
        self.assertTrue(file_obj2_UID in container.categorized_elements)

        # copy/paste the container
        copied_data = self.portal.manage_copyObjects(ids=[container.getId()])
        infos = self.portal.manage_pasteObjects(copied_data)
        new_container = self.portal[infos[0]['new_id']]
        self.assertEquals(len(new_container.categorized_elements), 2)
        # old no more referenced
        self.assertTrue(file_obj1_UID not in new_container.categorized_elements)
        self.assertTrue(file_obj2_UID not in new_container.categorized_elements)
        # copied contents are correctly referenced
        copied_file1 = new_container['file1']
        copied_file2 = new_container['file2']
        self.assertTrue(copied_file1.UID() in new_container.categorized_elements)
        self.assertTrue(copied_file2.UID() in new_container.categorized_elements)

    def test_defer_categorized_content_created_event(self):
        """Test that when defering management of the entire
           categorized_content_created event."""
        # not defered
        container = api.content.create(
            id='folder',
            type='Folder',
            container=self.portal
        )
        self.portal.REQUEST.set('defer_categorized_content_created_event', False)
        file_obj1 = api.content.create(
            id='file1',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        file_obj2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        self.assertTrue(file_obj1.UID() in container.categorized_elements)
        self.assertTrue(file_obj2.UID() in container.categorized_elements)

        # defered defer_categorized_content_created_event
        container2 = api.content.create(
            id='folder2',
            type='Folder',
            container=self.portal
        )
        self.portal.REQUEST.set('defer_categorized_content_created_event', True)
        file_obj3 = api.content.create(
            id='file3',
            type='File',
            file=self.file,
            container=container2,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        file_obj4 = api.content.create(
            id='file4',
            type='File',
            file=self.file,
            container=container2,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        self.assertFalse(base_hasattr(container2, 'categorized_elements'))
        # calling utils.update_all_categorized_elements will update necessary things
        utils.update_all_categorized_elements(container2)
        self.assertTrue(file_obj3.UID() in container2.categorized_elements)
        self.assertTrue(file_obj4.UID() in container2.categorized_elements)
        # tear down
        self.portal.REQUEST.set('defer_categorized_content_created_event', False)
        self.portal.REQUEST.set('defer_update_categorized_elements', False)

    def test_defer_update_categorized_elements(self):
        """Using 'defer_update_categorized_elements' will avoid the call to
           utils.update_all_categorized_elements on categorized element create/update."""
        # defered defer_update_categorized_elements
        container = api.content.create(
            id='folder',
            type='Folder',
            container=self.portal
        )
        self.portal.REQUEST.set('defer_update_categorized_elements', True)
        file_obj1 = api.content.create(
            id='file1',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        file_obj2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        self.assertFalse(base_hasattr(container, 'categorized_elements'))
        # calling utils.update_all_categorized_elements will update necessary things
        utils.update_all_categorized_elements(container)
        self.assertTrue(file_obj1.UID() in container.categorized_elements)
        self.assertTrue(file_obj2.UID() in container.categorized_elements)
        # tear down
        self.portal.REQUEST.set('defer_categorized_content_created_event', False)
        self.portal.REQUEST.set('defer_update_categorized_elements', False)

    def test_defer_update_categorized_elements_when_cloned(self):
        """Call to 'update_all_categorized_elements' may be defered
           when cloning a categorized elements container."""
        container = api.content.create(
            id='folder',
            type='Folder',
            container=self.portal
        )
        api.content.create(
            id='file1',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=container,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        # clone container
        self.portal.REQUEST.set('defer_update_categorized_elements', True)
        copy_info = self.portal.manage_copyObjects(ids=[container.getId()])
        paste_infos = self.portal.manage_pasteObjects(copy_info)
        new_container = self.portal.get(paste_infos[0]['new_id'])
        new_file_obj1 = new_container.file1
        new_file_obj2 = new_container.file2
        self.assertFalse(new_file_obj1.UID() in new_container.categorized_elements)
        self.assertFalse(new_file_obj2.UID() in new_container.categorized_elements)
        # calling utils.update_all_categorized_elements will update necessary things
        utils.update_all_categorized_elements(new_container)
        self.assertTrue(new_file_obj1.UID() in new_container.categorized_elements)
        self.assertTrue(new_file_obj2.UID() in new_container.categorized_elements)
        self.assertEqual(len(new_container.categorized_elements), 2)
        # tear down
        self.portal.REQUEST.set('defer_update_categorized_elements', False)

    def test_delete_categorized_element(self):
        """When an element having a content_category is deleted, the parent's
           categorized_elements are update accordingly."""
        file_UID = self.portal.file_txt.UID()
        image_UID = self.portal.image.UID()
        self.assertTrue(file_UID in self.portal.categorized_elements)
        self.assertTrue(image_UID in self.portal.categorized_elements)

        # remove self.portal.file, will be removed from parent's categorized_elements
        api.content.delete(self.portal.file_txt)
        self.assertFalse(file_UID in self.portal.categorized_elements)

        # remove self.portal.image
        # does not break if was not in parent's categorized_elements
        self.portal.categorized_elements.clear()
        self.assertFalse(image_UID in self.portal.categorized_elements)
        api.content.delete(self.portal.image)
        self.assertFalse(image_UID in self.portal.categorized_elements)

        # does not break if parent does not have a categorized_elements attribute
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
        )
        delattr(self.portal, 'categorized_elements')
        api.content.delete(file2)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.documentviewer.config import CONVERTABLE_TYPES
from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory import testing
from collective.iconifiedcategory.behaviors.iconifiedcategorization import IIconifiedCategorization
from collective.iconifiedcategory.tests.base import BaseTestCase
from collective.iconifiedcategory.utils import calculate_category_id
from collective.iconifiedcategory.utils import get_category_object
from plone import api
from z3c.form import validator
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from ZPublisher.HTTPRequest import FileUpload

import cgi
import unittest


class TestIconifiedCategorization(BaseTestCase, unittest.TestCase):
    layer = testing.COLLECTIVE_ICONIFIED_CATEGORY_FUNCTIONAL_TESTING

    def test_content_category_edit_form(self):
        """ """
        # displayed on the edit form
        edit = self.portal.file_txt.restrictedTraverse('@@edit')
        edit.update()
        rendered_edit = edit()
        self.assertTrue('IIconifiedCategorization.content_category' in rendered_edit)
        self.assertTrue('IIconifiedCategorization.default_titles' in rendered_edit)

    def test_content_category_not_set_if_not_activated(self):
        """ """
        category_group = self.portal.config['group-1']
        category = self.portal.config['group-1']['category-1-1']
        content_category_id = calculate_category_id(category)
        category_group.to_be_printed_activated = False
        category_group.confidentiality_activated = False
        category.to_print = True
        category.confidential = True
        obj = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertFalse(obj.to_print)
        self.assertFalse(obj.confidential)

    def test_content_category_confidential_on_creation(self):
        """ """
        category_group = self.portal.config['group-1']
        category = self.portal.config['group-1']['category-1-1']
        content_category_id = calculate_category_id(category)
        category_group.confidentiality_activated = True

        # set to False
        category.confidential = False
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertFalse(file2.confidential)

        # set to True
        category.confidential = True
        file3 = api.content.create(
            id='file3',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertTrue(file3.confidential)

    def test_content_category_to_print_on_creation(self):
        """ """
        category_group = self.portal.config['group-1']
        category = self.portal.config['group-1']['category-1-1']
        content_category_id = calculate_category_id(category)
        category_group.to_be_printed_activated = True
        # enable conversion
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()

        # set to False
        category.to_print = False
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertFalse(file2.to_print)

        # set to True
        category.to_print = True
        file3 = api.content.create(
            id='file3',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertTrue(file3.to_print)

    def test_content_category_to_sign_signed_on_creation(self):
        """ """
        category_group = self.portal.config['group-1']
        category = self.portal.config['group-1']['category-1-1']
        content_category_id = calculate_category_id(category)
        category_group.signed_activated = True

        # set to False both attributes
        category.to_sign = False
        category.signed = False
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertFalse(file2.to_sign)
        self.assertFalse(file2.signed)

        # set True for to_sign, False for signed
        category.to_sign = True
        category.signed = False
        file3 = api.content.create(
            id='file3',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertTrue(file3.to_sign)
        self.assertFalse(file3.signed)

        # set True for both attributes
        category.to_sign = True
        category.signed = True
        file4 = api.content.create(
            id='file4',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        self.assertTrue(file4.to_sign)
        self.assertTrue(file4.signed)

    def test_content_category_to_print_only_set_if_convertible_when_conversion_enabled(self):
        """ """
        category_group = self.portal.config['group-1']
        category = self.portal.config['group-1']['category-1-1']
        content_category_id = calculate_category_id(category)
        category_group.to_be_printed_activated = True

        # set to True
        category.to_print = True
        file2 = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        # enable conversion
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        file2.file.contentType = 'text/unknown'

        notify(ObjectModifiedEvent(file2))
        self.assertIsNone(file2.to_print)

    def test_content_category_setter_reindex_content_category_uid(self):
        """ """
        catalog = api.portal.get_tool('portal_catalog')
        obj = self.portal['file_txt']
        category = get_category_object(obj, obj.content_category)
        # correctly indexed on creation
        category_brain = catalog(content_category_uid=category.UID())[0]
        self.assertEqual(category_brain.UID, obj.UID())
        obj_brain = catalog(UID=obj.UID())[0]
        self.assertEqual(obj_brain.content_category_uid, category.UID())
        # correctly reindexed when content_category changed thru setter
        category2 = self.portal.config['group-1']['category-1-2']
        self.assertNotEqual(category, category2)
        adapted_obj = IIconifiedCategorization(obj)
        setattr(adapted_obj, 'content_category', 'config_-_group-1_-_category-1-2')
        category2_brain = catalog(content_category_uid=category2.UID())[0]
        self.assertEqual(category2_brain.UID, obj.UID())
        obj_brain = catalog(UID=obj.UID())[0]
        self.assertEqual(obj_brain.content_category_uid, category2.UID())

    def test_content_category_changed_default_values(self):
        """While content_category is changed on an element, the default values for fields
           to_print/confidential/to_sign/signed are reapplied with new content_category
           default values if it was still the default value of the original content_category."""
        category_group = self.portal.config['group-1']
        category_group.to_be_printed_activated = True
        category_group.confidentiality_activated = True
        category_group.signed_activated = True
        category11 = self.portal.config['group-1']['category-1-1']
        category11_id = calculate_category_id(category11)
        obj = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=category11_id)
        self.assertFalse(obj.confidential)
        self.assertFalse(obj.to_sign)
        self.assertFalse(obj.signed)
        self.assertIsNone(obj.to_print)
        # now enable everything on category-1-2 and use it
        category12 = self.portal.config['group-1']['category-1-2']
        category12.to_print = False
        category12.confidential = True
        category12.to_sign = True
        category12.signed = True
        category12_id = calculate_category_id(category12)
        adapted_obj = IIconifiedCategorization(obj)
        setattr(adapted_obj, 'content_category', category12_id)
        notify(ObjectModifiedEvent(obj))

        # changed to True on obj an in parent's categorized_elements
        parent_cat_elements = obj.aq_parent.categorized_elements[obj.UID()]
        self.assertTrue(obj.confidential)
        self.assertTrue(parent_cat_elements['confidential'])
        self.assertTrue(obj.to_sign)
        self.assertTrue(parent_cat_elements['to_sign'])
        self.assertTrue(obj.signed)
        self.assertTrue(parent_cat_elements['signed'])
        # to_print could not be set to False because not printable
        self.assertIsNone(obj.to_print)
        self.assertIsNone(parent_cat_elements['to_print'])

        # enable conversion and back to category11
        category11.to_print = True
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        # set to_print to False, aka the default value of category12
        obj.to_print = False
        setattr(adapted_obj, 'content_category', category11_id)
        notify(ObjectModifiedEvent(obj))
        self.assertTrue(obj.to_print)
        self.assertTrue(parent_cat_elements['to_print'])

        # if original content_category does not exist, it does not fail
        # but default values are not reapplied
        obj.content_category = 'unkown_content_category'
        setattr(adapted_obj, 'content_category', category12_id)
        notify(ObjectModifiedEvent(obj))
        self.assertEqual(obj.content_category, category12_id)

    def test_unexisting_content_category(self):
        """Storing an unexisting content_category does not break anything."""
        obj = api.content.create(
            id='my-file',
            type='File',
            file=self.file,
            container=self.portal,
            content_category='unexisting_content_category_uid')
        self.assertTrue(obj)

    def test_only_pdf_invariant(self):
        """When category.only_pdf is True, categorized file may only be a PDF file."""
        category = self.portal.config['group-1']['category-1-1']
        category.only_pdf = False
        content_category_id = calculate_category_id(category)
        invariants = validator.InvariantsValidator(
            None, None, None, IIconifiedCategorization, None)
        # form and data in request
        fieldstorage = cgi.FieldStorage()
        fieldstorage.file = self.file.data
        fieldstorage.filename = self.file.filename
        data = {}
        data['content_category'] = content_category_id
        request = self.portal.REQUEST
        request.form['form.widgets.file'] = FileUpload(fieldstorage)
        view = self.portal.restrictedTraverse('folder_contents')
        request['PUBLISHED'] = view
        self.assertFalse(invariants.validate(data))
        # already_validateFileIsPDF was set in REQUEST
        self.assertTrue(request.get('already_validateFileIsPDF'))
        self.assertFalse(invariants.validate(data))
        # PDF needed
        request.set('already_validateFileIsPDF', False)
        category.only_pdf = True
        errors = invariants.validate(data)
        request.set('already_validateFileIsPDF', False)
        self.assertEqual(errors[0].message, u'You must select a PDF file!')
        request.form['form.widgets.file'].headers['content-type'] = 'application/pdf'
        # no file in request
        request.form = {}
        self.assertFalse(invariants.validate(data))
        request.set('already_validateFileIsPDF', False)
        # editing a stored element
        category.only_pdf = False
        obj = api.content.create(
            id='file2',
            type='File',
            file=self.file,
            container=self.portal,
            content_category=content_category_id)
        view = obj.restrictedTraverse('view')
        request['PUBLISHED'] = view
        self.assertFalse(invariants.validate(data))
        request.set('already_validateFileIsPDF', False)
        # PDF needed
        category.only_pdf = True
        errors = invariants.validate(data)
        request.set('already_validateFileIsPDF', False)
        self.assertEqual(errors[0].message, u'You must select a PDF file!')
        obj.file = self.file_pdf
        self.assertFalse(invariants.validate(data))



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone.app.contenttypes.interfaces import IFolder
from plone.dexterity.content import Container
from zope.interface import implements


class ICategoryConfiguration(IFolder):
    """Marker interface of ContentCategoryConfiguration"""


class CategoryConfiguration(Container):
    implements(ICategoryConfiguration)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from plone.app.registry.browser import controlpanel


class IconifiedCategorySettingsEditForm(controlpanel.RegistryEditForm):
    schema = IIconifiedCategorySettings
    label = _(u'Iconified Category Settings')


class IconifiedCategorySettingsView(controlpanel.ControlPanelFormWrapper):
    form = IconifiedCategorySettingsEditForm



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from AccessControl import Unauthorized
from collective.iconifiedcategory import _
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.event import IconifiedAttrChangedEvent
from collective.iconifiedcategory.interfaces import IIconifiedPrintable
from plone import api
from Products.CMFCore.permissions import ModifyPortalContent
from Products.Five import BrowserView
from z3c.json.interfaces import IJSONWriter
from zope.component import getAdapter
from zope.component import getUtility
from zope.event import notify
from zope.i18n import translate


class BaseView(BrowserView):
    attribute_mapping = {}
    category_group_attr_name = ''
    attr_name = ''
    # when updating element, do that limited?
    limited = True

    def _translate(self, msgid):
        return translate(
            msgid,
            domain='collective.iconifiedcategory',
            context=self.request,
        )

    def __call__(self):
        """Do the work :
           - status :
               -1 --> deactivate;
               0 --> set to False;
               1 --> set to True;
               2 --> error;
           """
        writer = getUtility(IJSONWriter)
        values = {'msg': u'Values have been set'}
        try:
            self.request.response.setHeader('content-type',
                                            'application/json')
            status, msg = self.set_values(self.get_values())
            values['status'] = status
            if msg:
                values['msg'] = self._translate(msg)
        except Exception:
            values['status'] = 2
            values['msg'] = self._translate(_('Error during process'))
        return writer.write(values)

    def get_current_values(self):
        return {k: getattr(self.context, k)
                for k in self.attribute_mapping.keys()}

    def get_values(self):
        return {k: self.convert_boolean(self.request.get(v))
                for k, v in self.attribute_mapping.items()}

    def _may_set_values(self, values, ):
        res = bool(api.user.has_permission(ModifyPortalContent, obj=self.context))
        if res:
            # is this functionnality enabled?
            self.category = utils.get_category_object(self.context, self.context.content_category)
            category_group = self.category.get_category_group()
            res = getattr(category_group, self.category_group_attr_name, True)
        return res

    def set_values(self, values):
        if not self._may_set_values(values):
            raise Unauthorized

        if not values:
            return 2, self._translate(_('No values to set'))

        old_values = self.get_current_values()

        for key, value in values.items():
            self._set_value(key, value)
        status, msg = self._get_status(values), utils.boolean_message(
            self.context, attr_name=self.attr_name)
        if not status == 2:
            utils.update_categorized_elements(
                self.context.aq_parent,
                self.context,
                self.category,
                limited=self.limited,
                sort=False)
            notify(IconifiedAttrChangedEvent(
                self.context,
                self.attr_name,
                old_values,
                values,
            ))
        return status, msg

    def _get_status(self, values):
        value = values.get(self.attribute_mapping.keys()[0], None)
        if value is False:
            return 0
        elif value is True:
            return 1
        else:
            return -1

    def _set_value(self, attrname, value):
        setattr(self.context, attrname, value)

    @staticmethod
    def convert_boolean(value):
        values = {
            'false': False,
            'true': True,
        }
        return values.get(value, value)


class ToPrintChangeView(BaseView):
    attribute_mapping = {
        'to_print': 'iconified-value',
    }
    category_group_attr_name = 'to_be_printed_activated'
    attr_name = 'to_print'

    def set_values(self, values):
        status, msg = super(ToPrintChangeView, self).set_values(values)
        adapter = getAdapter(self.context, IIconifiedPrintable)
        adapter.update_object()
        return status, utils.print_message(self.context)


class ConfidentialChangeView(BaseView):
    attribute_mapping = {
        'confidential': 'iconified-value',
    }
    category_group_attr_name = 'confidentiality_activated'
    attr_name = 'confidential'
    limited = False


class SignedChangeView(BaseView):
    attribute_mapping = {
        'signed': 'iconified-value',
        'to_sign': 'iconified-value',
    }
    category_group_attr_name = 'signed_activated'
    attr_name = 'to_sign'

    def _get_next_values(self, old_values):
        """ """
        values = {}
        if old_values['to_sign'] is False:
            values['to_sign'] = True
            values['signed'] = False
            status = 0
        elif old_values['to_sign'] is True and old_values['signed'] is False:
            values['to_sign'] = True
            values['signed'] = True
            status = 1
        else:
            # old_values['to_sign'] is True and old_values['signed'] is True
            # disable to_sign and signed
            values['to_sign'] = False
            values['signed'] = False
            status = -1
        return status, values

    def set_values(self, values):
        """Value are setting 'to_print' and 'signed' attributes with following
           possibility depending on allowed ones :
           - False/False;
           - True/False;
           - True/True."""
        old_values = self.get_current_values()
        status, values = self._get_next_values(old_values)
        super(SignedChangeView, self).set_values(values)
        return status, utils.signed_message(self.context)


class PublishableChangeView(BaseView):
    attribute_mapping = {
        'publishable': 'iconified-value',
    }
    category_group_attr_name = 'publishable_activated'
    attr_name = 'publishable'



# -*- coding: utf-8 -*-
from collective.iconifiedcategory import logger
from collective.iconifiedcategory.interfaces import IIconifiedInfos
from collective.iconifiedcategory.utils import update_all_categorized_elements
from plone import api
from plone.dexterity.fti import DexterityFTI
from Products.CMFPlone.utils import base_hasattr
from Products.ZCatalog.ProgressHandler import ZLogHandler
from zope.component import getAdapter

import transaction


behavior_id = 'collective.iconifiedcategory.behaviors.iconifiedcategorization.IIconifiedCategorization'


def _portal_types_using_behavior():
    ''' '''
    types_tool = api.portal.get_tool('portal_types')
    portal_types = []
    for type_info in types_tool.listTypeInfo():
        if isinstance(type_info, DexterityFTI) and behavior_id in type_info.behaviors:
            portal_types.append(type_info.id)
    return portal_types


def upgrade_to_2100(context):
    '''
    '''
    # get every categories and generate scales for it or it is generated at first access
    brains = api.content.find(object_provides='collective.iconifiedcategory.content.category.ICategory')
    for brain in brains:
        category = brain.getObject()
        category.restrictedTraverse('@@images').scale(scale='listing')
    # commit so scales are really available when updating categorized elements here under
    transaction.commit()

    # get portal_types using IIconifiedCategorization behavior
    portal_types = _portal_types_using_behavior()
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(portal_type=portal_types)

    logger.info('Querying elements to update among "{0}" objects of portal_type "{1}"'.format(
        len(brains), ', '.join(portal_types)))
    parents_to_update = []
    for brain in brains:
        obj = brain.getObject()
        # this can be useless if using behavior 'Scan metadata' collective.dms.scanbehavior
        if not(base_hasattr(obj, 'to_sign')):
            setattr(obj, 'to_sign', False)
        if not(base_hasattr(obj, 'signed')):
            setattr(obj, 'signed', False)

        parent = obj.aq_parent
        if parent not in parents_to_update:
            parents_to_update.append(parent)

    # finally update parents that contains categorized elements
    nb_of_parents_to_update = len(parents_to_update)
    i = 1
    for parent_to_update in parents_to_update:
        logger.info('Running update_all_categorized_elements for element {0}/{1} ({2})'.format(
            i, nb_of_parents_to_update, '/'.join(parent_to_update.getPhysicalPath())))
        i = i + 1
        # recompute everything including sorting
        update_all_categorized_elements(parent_to_update)


def upgrade_to_2101(context):
    ''' '''
    # get portal_types using IIconifiedCategorization behavior
    portal_types = _portal_types_using_behavior()
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(portal_type=portal_types)

    logger.info('Querying elements to update among "{0}" objects of portal_type "{1}"'.format(
        len(brains), ', '.join(portal_types)))
    parents_to_update = []
    for brain in brains:
        obj = brain.getObject()
        if not(base_hasattr(obj, 'publishable')):
            setattr(obj, 'publishable', False)

        parent = obj.aq_parent
        if parent not in parents_to_update:
            parents_to_update.append(parent)

    # finally update parents that contains categorized elements
    nb_of_parents_to_update = len(parents_to_update)
    i = 1
    for parent_to_update in parents_to_update:
        logger.info('Running update_all_categorized_elements for element {0}/{1} ({2})'.format(
            i, nb_of_parents_to_update, '/'.join(parent_to_update.getPhysicalPath())))
        i = i + 1
        update_all_categorized_elements(parent_to_update)


def upgrade_to_2102(context):
    '''Now that elements using the behavior are marked with IIconifiedCategorizationMarker,
       reindex object_provides to get elements to update.
       Then compute "allowedRolesAndUsers" for categorized elements.
     '''
    catalog = api.portal.get_tool('portal_catalog')
    # update object_provides index
    pghandler = ZLogHandler(steps=1000)
    catalog.reindexIndex('object_provides', context.REQUEST, pghandler=pghandler)

    # compute allowedRolesAndUsers for categorized elements
    brains = catalog(
        object_provides='collective.iconifiedcategory.'
        'behaviors.iconifiedcategorization.IIconifiedCategorizationMarker')
    i = 0
    pghandler = ZLogHandler(steps=1000)
    pghandler.info('Computing "allowedRolesAndUsers" for categorized elements...')
    pghandler.init('ComputeAllowedRolesAndUsers', len(brains))

    for brain in brains:
        i += 1
        pghandler.report(i)
        obj = brain.getObject()
        adapter = getAdapter(obj, IIconifiedInfos)
        allowedRolesAndUsers = adapter._allowedRolesAndUsers
        parent = obj.aq_inner.aq_parent
        parent.categorized_elements[obj.UID()]['allowedRolesAndUsers'] = allowedRolesAndUsers
        parent._p_changed = True
    pghandler.finish()


def upgrade_to_2103(context):
    '''The element "last_updated" is now in categorized_elements.'''
    catalog = api.portal.get_tool('portal_catalog')

    # compute "last_updated" for categorized elements
    brains = catalog(
        object_provides='collective.iconifiedcategory.'
        'behaviors.iconifiedcategorization.IIconifiedCategorizationMarker')
    i = 0
    pghandler = ZLogHandler(steps=1000)
    pghandler.info('Computing "last_updated" for categorized elements...')
    pghandler.init('ComputeLastUpdatedForCategorizedElements', len(brains))

    for brain in brains:
        i += 1
        pghandler.report(i)
        obj = brain.getObject()
        parent = obj.aq_inner.aq_parent
        modified = obj.modified()
        modified._timezone_naive = True
        if 'last_updated' in parent.categorized_elements[obj.UID()]:
            # already upgraded
            continue
        parent.categorized_elements[obj.UID()]['last_updated'] = modified.asdatetime()
        parent._p_changed = True
    pghandler.finish()


def upgrade_to_2104(context):
    '''Add info "show_preview" to categorized_elements.'''
    catalog = api.portal.get_tool('portal_catalog')

    brains = catalog(
        object_provides='collective.iconifiedcategory.'
        'behaviors.iconifiedcategorization.IIconifiedCategorizationMarker')
    i = 0
    pghandler = ZLogHandler(steps=1000)
    pghandler.info('Initializing "show_preview" to False for every categorized elements...')
    pghandler.init('InitShowPreviewForCategorizedElements', len(brains))

    for brain in brains:
        i += 1
        pghandler.report(i)
        obj = brain.getObject()
        parent = obj.aq_inner.aq_parent
        obj_uid = obj.UID()
        if 'show_preview' in parent.categorized_elements[obj_uid]:
            # already upgraded
            continue
        parent.categorized_elements[obj_uid]['show_preview'] = 0
        parent._p_changed = True
    pghandler.finish()


def upgrade_to_2105(context):
    '''Add info "contentType" to categorized_elements.'''
    catalog = api.portal.get_tool('portal_catalog')

    brains = catalog(
        object_provides='collective.iconifiedcategory.'
        'behaviors.iconifiedcategorization.IIconifiedCategorizationMarker')
    i = 0
    pghandler = ZLogHandler(steps=1000)
    pghandler.info('Initializing "contentType" for every categorized elements...')
    pghandler.init('InitFileContentTypeForCategorizedElements', len(brains))

    for brain in brains:
        i += 1
        pghandler.report(i)
        obj = brain.getObject()
        parent = obj.aq_inner.aq_parent
        obj_uid = obj.UID()
        if 'contentType' in parent.categorized_elements[obj_uid]:
            # already upgraded
            continue
        adapter = getAdapter(obj, IIconifiedInfos)
        parent.categorized_elements[obj_uid]['contentType'] = adapter._file_contentType
        parent._p_changed = True
    pghandler.finish()



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_base
from collective.iconifiedcategory import _
from collective.iconifiedcategory.utils import get_category_object
from collective.iconifiedcategory.utils import validateFileIsPDF
from collective.iconifiedcategory.widget.widget import CategoryTitleFieldWidget
from collective.z3cform.select2.widget.widget import SingleSelect2FieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import invariant
from zope.interface import provider


@provider(IFormFieldProvider)
class IIconifiedCategorization(Interface):

    form.order_before(content_category='title')
    form.order_before(content_category='IBasic.title')
    form.order_before(content_category='IDublinCore.title')
    form.widget(content_category=SingleSelect2FieldWidget)
    content_category = schema.Choice(
        title=_(u'Category'),
        source='collective.iconifiedcategory.categories',
        required=True,
    )

    form.order_before(default_titles='title')
    form.order_before(default_titles='IBasic.title')
    form.order_before(default_titles='IDublinCore.title')
    form.mode(default_titles='hidden')
    form.widget(default_titles=CategoryTitleFieldWidget)
    default_titles = schema.Choice(
        title=_(u'Default title'),
        vocabulary='collective.iconifiedcategory.category_titles',
        required=False,
    )

    @invariant
    def validateFileIsPDFInvariant(data):
        validateFileIsPDF(data)


class IIconifiedCategorizationMarker(Interface):
    """ """


@implementer(IIconifiedCategorization)
@adapter(IIconifiedCategorizationMarker)
class IconifiedCategorization(object):

    def __init__(self, context):
        self.context = context

    @property
    def default_titles(self):
        return None

    @property
    def content_category(self):
        return getattr(self.context, 'content_category', None)

    def _content_category_changed_default_values(self, new_value):
        """When changing content_category, change default values
           if original default was not yet changed."""
        try:
            current_category = get_category_object(
                self.context, self.context.content_category)
            new_category = get_category_object(
                self.context, new_value)
        except KeyError:
            # in case we can not get the category, we return
            # this can be the case when changing category of an element
            # and old category does not exist in the content_category_grouyp context
            return
        category_group = current_category.get_category_group(current_category)
        # to_print
        if category_group.to_be_printed_activated and \
           self.context.to_print == current_category.to_print:
            self.context.to_print = new_category.to_print
        # confidential
        if category_group.confidentiality_activated and \
           self.context.confidential == current_category.confidential:
            self.context.confidential = new_category.confidential
        # to_sign/signed
        if category_group.signed_activated and \
           self.context.to_sign == current_category.to_sign and \
           self.context.signed == current_category.signed:
            self.context.to_sign = new_category.to_sign
            self.context.signed = new_category.signed
        # publishable
        if category_group.publishable_activated and \
           self.context.publishable == current_category.publishable:
            self.context.publishable = new_category.publishable

    @content_category.setter
    def content_category(self, value):
        # if content_category changed, we check also if default values
        # need to be updated.  We will update values that were not modified
        # since last default values
        if getattr(self.context, 'content_category', None) and self.context.content_category != value:
            self._content_category_changed_default_values(value)
        self.context.content_category = value
        self.context.reindexObject(idxs=['content_category_uid'])

    @property
    def to_print(self):
        return getattr(aq_base(self.context), 'to_print', False)

    @property
    def confidential(self):
        return getattr(aq_base(self.context), 'confidential', False)

    @property
    def to_sign(self):
        return getattr(aq_base(self.context), 'to_sign', False)

    @property
    def signed(self):
        return getattr(aq_base(self.context), 'signed', False)

    @property
    def publishable(self):
        return getattr(aq_base(self.context), 'publishable', False)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import utils
from Products.Five import BrowserView


class IconifiedCategory(BrowserView):

    def __call__(self, *args, **kwargs):
        self.request.response.setHeader('Content-Type', 'text/css')
        content = []
        css = (u".{0} {{ padding-left: 1.4em; background: "
               u"transparent url('{1}') no-repeat top left; "
               u"background-size: contain; }}")
        if utils.has_config_root(self.context) is False:
            return ''
        # sort_on=None to avoid useless sort_on="getObjPositionInParent"
        categories = utils.get_categories(self.context,
                                          sort_on=None,
                                          only_enabled=False)
        for category in categories:
            obj = category._unrestrictedGetObject()
            category_id = utils.calculate_category_id(obj)
            url = u'{0}/@@download'.format(obj.absolute_url())
            content.append(css.format(utils.format_id_css(category_id), url))
        return ' '.join(content)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory.content.categorygroup import ICategoryGroup
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.browser.add import DefaultAddView
from plone.dexterity.browser.edit import DefaultEditForm
from plone.dexterity.browser.view import DefaultView
from plone.dexterity.interfaces import IDexterityEditForm
from plone.z3cform import layout
from z3c.form.interfaces import HIDDEN_MODE
from zope.interface import classImplements


class FormMixin(object):
    related_widgets = {
        'confidential': 'confidentiality_activated',
        'to_print': 'to_be_printed_activated',
        'to_sign': 'signed_activated',
        'signed': 'signed_activated',
    }

    @property
    def category_group(self):
        if ICategoryGroup.providedBy(self.context):
            return self.context
        return self.context.get_category_group()

    def updateWidgets(self):
        super(FormMixin, self).updateWidgets()
        for name, widget in self.widgets.items():
            related_attribute = self.related_widgets.get(name)
            if not related_attribute:
                continue
            if getattr(self.category_group, related_attribute) is False:
                widget.mode = HIDDEN_MODE


class BaseEditForm(FormMixin, DefaultEditForm):
    pass


BaseEditView = layout.wrap_form(BaseEditForm)
classImplements(BaseEditView, IDexterityEditForm)


class BaseAddForm(FormMixin, DefaultAddForm):
    pass


class BaseAddView(DefaultAddView):
    form = BaseAddForm


class BaseView(DefaultView):
    related_widgets = {
        'confidential': 'confidentiality_activated',
        'to_print': 'to_be_printed_activated',
        'to_sign': 'signed_activated',
        'signed': 'signed_activated',
    }

    def updateWidgets(self):
        super(BaseView, self).updateWidgets()
        for rel_widget, rel_attribute in self.related_widgets.items():
            parent_value = getattr(
                self.context.get_category_group(),
                rel_attribute,
                None,
            )
            if parent_value is False and rel_widget in self.widgets:
                del self.widgets[rel_widget]



# -*- coding: utf-8 -*-

from collections import OrderedDict
from collective.documentviewer.config import CONVERTABLE_TYPES
from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.browser.tabview import CategorizedContent
from collective.iconifiedcategory.browser.tabview import PrintColumn
from collective.iconifiedcategory.tests.base import BaseTestCase
from plone import api
from Products.CMFCore.permissions import ModifyPortalContent
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent


class TestCategorizedTabView(BaseTestCase):

    def test_table_render(self):
        view = self.portal.restrictedTraverse('@@iconifiedcategory')
        result = view()
        self.assertTrue('<a href="http://nohost/plone/file_txt" ' in result)
        self.assertTrue('File description' in result)
        self.assertTrue('<a href="http://nohost/plone/image" ' in result)
        self.assertTrue('Image description' in result)
        self.assertTrue('<td class="td_cell_category-column">Category 1-1</td>' in result)

        # 'to_print' and 'confidential' related columns are displayed by default
        self.assertTrue('<th>To be printed</th>' in result)
        self.assertTrue('<th>Confidential</th>' in result)

        # when nothing to display
        api.content.delete(self.portal['file_txt'])
        api.content.delete(self.portal['image'])
        self.assertEqual(self.portal.categorized_elements, OrderedDict())
        self.assertTrue('No element to display.' in view())

    def test_table_render_special_chars(self):
        """Special chars used in :
           - element's title;
           - category title;
           - subcategory title."""
        category = api.content.create(
            type='ContentCategory',
            title='Category \xc3\xa0',
            icon=self.icon,
            container=self.portal.config['group-1'],
        )
        subcategory = api.content.create(
            type='ContentSubcategory',
            title='Subcategory \xc3\xa0',
            icon=self.icon,
            container=category,
        )
        document = api.content.create(
            type='Document',
            title='Document \xc3\xa0',
            container=self.portal,
            content_category=utils.calculate_category_id(category),
            to_print=False,
            confidential=False,
        )
        document2 = api.content.create(
            type='Document',
            title='Document \xc3\xa0',
            container=self.portal,
            content_category=utils.calculate_category_id(subcategory),
            to_print=False,
            confidential=False,
        )
        # make sure every elements are correctly registered in categorized_elements
        # it seems that registration is made before element's id is renamed...
        utils.update_all_categorized_elements(self.portal)
        view = self.portal.restrictedTraverse('@@iconifiedcategory')
        result = view()
        self.assertTrue(category.title in result)
        self.assertTrue(subcategory.title in result)
        self.assertTrue(document.title in result)
        self.assertTrue(document2.title in result)

    def test_table_render_when_preview_enabled(self):
        self.portal.portal_properties.site_properties.typesUseViewActionInListings = ()
        # enable collective.documentviewer so document is convertible
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        # initialize collective.documentviewer annotations on file
        file_obj = self.portal['file_txt']
        image_obj = self.portal['image']
        notify(ObjectModifiedEvent(file_obj))
        notify(ObjectModifiedEvent(image_obj))

        view = self.portal.restrictedTraverse('@@iconifiedcategory')
        result = view()
        # by default, images are not handled by collective.documentviewer
        self.assertTrue('<a href="http://nohost/plone/image" ' in result)
        self.assertTrue('<a href="http://nohost/plone/file_txt/documentviewer#document/p1" ' in result)

    def test_PrintColumn(self):
        table = self.portal.restrictedTraverse('@@iconifiedcategory')
        file_infos = utils.get_categorized_elements(
            self.portal, uids=[self.portal['file_txt'].UID()])
        categorized_content = CategorizedContent(self.portal, file_infos[0])
        obj = categorized_content.getObject()
        column = PrintColumn(self.portal, self.portal.REQUEST, table)
        # not convertible by default as c.documentviewer not enabled
        self.assertEqual(
            column.renderCell(categorized_content),
            u'<a href="#" '
            u'class="iconified-action deactivated" '
            u'alt="Not convertible to a printable format" '
            u'title="Not convertible to a printable format"></a>')
        self.assertIsNone(categorized_content.to_print)
        self.assertIsNone(obj.to_print)

        # enable collective.documentviewer so document is convertible
        gsettings = GlobalSettings(self.portal)
        gsettings.auto_layout_file_types = CONVERTABLE_TYPES.keys()
        # enable to_print management in configuration
        category = utils.get_category_object(obj, obj.content_category)
        category_group = category.get_category_group(category)
        category_group.to_be_printed_activated = True
        category.to_print = False
        notify(ObjectModifiedEvent(obj))
        file_infos = utils.get_categorized_elements(
            self.portal, uids=[self.portal['file_txt'].UID()])
        categorized_content = CategorizedContent(self.portal, file_infos[0])
        self.assertFalse(categorized_content.to_print)
        self.assertFalse(obj.to_print)
        self.assertEqual(
            column.renderCell(categorized_content),
            u'<a href="http://nohost/plone/file_txt/@@iconified-print" '
            u'class="iconified-action editable" '
            u'alt="Should not be printed" '
            u'title="Should not be printed"></a>')

        # set to_print to True
        obj.to_print = True
        notify(ObjectModifiedEvent(obj))
        file_infos = utils.get_categorized_elements(
            self.portal, uids=[self.portal['file_txt'].UID()])
        categorized_content = CategorizedContent(self.portal, file_infos[0])
        self.assertTrue(categorized_content.to_print)
        self.assertTrue(obj.to_print)
        self.assertEqual(
            column.renderCell(categorized_content),
            u'<a href="http://nohost/plone/file_txt/@@iconified-print" '
            u'class="iconified-action active editable" '
            u'alt="Must be printed" '
            u'title="Must be printed"></a>')

        # if element is not editable, the 'editable' CSS class is not there
        obj.manage_permission(ModifyPortalContent, roles=[])
        notify(ObjectModifiedEvent(obj))
        self.assertEqual(column.renderCell(categorized_content),
                         u'<a href="http://nohost/plone/file_txt/@@iconified-print" '
                         u'class="iconified-action active" '
                         u'alt="Must be printed" title="Must be printed"></a>')



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _
from plone.autoform import directives as form
from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import Interface
from zope.interface import Invalid
from zope.interface import invariant


class ICategorize(Interface):

    predefined_title = schema.TextLine(
        title=_(u'Predefined title'),
        required=False,
    )

    form.widget('confidential', RadioFieldWidget)
    confidential = schema.Bool(
        title=_(u'Confidential default'),
        required=False,
        default=False,
    )

    form.widget('to_print', RadioFieldWidget)
    to_print = schema.Bool(
        title=_(u'To be printed default'),
        required=False,
        default=False,
    )

    form.widget('to_sign', RadioFieldWidget)
    to_sign = schema.Bool(
        title=_(u'To sign default'),
        required=False,
        default=False,
    )

    form.widget('signed', RadioFieldWidget)
    signed = schema.Bool(
        title=_(u'Signed default'),
        required=False,
        default=False,
    )

    form.widget('publishable', RadioFieldWidget)
    publishable = schema.Bool(
        title=_(u'Publishable default'),
        required=False,
        default=False,
    )

    form.widget('enabled', RadioFieldWidget)
    enabled = schema.Bool(
        title=_(u'Enabled?'),
        default=True,
        required=False,
    )

    form.widget('only_pdf', RadioFieldWidget)
    only_pdf = schema.Bool(
        title=_(u'Only PDF?'),
        default=False,
        required=False,
    )

    show_preview = schema.Choice(
        title=_(u'Show preview?'),
        description=_(u'Show preview description.'),
        vocabulary='collective.iconifiedcategory.show_preview_vocabulary',
        default=0,
        required=True,
    )

    @invariant
    def signedInvariant(data):
        """'signed' may only be True if 'to_sign' is True."""
        if data.to_sign is False and data.signed is True:
            raise Invalid(_(u"'Signed' can not be True when 'To sign?' is False!"))



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _
from plone.app.contenttypes.interfaces import IFolder
from plone.autoform import directives as form
from plone.dexterity.content import Container
from plone.dexterity.schema import DexteritySchemaPolicy
from z3c.form.browser.radio import RadioFieldWidget
from zope import schema
from zope.interface import implements


class ICategoryGroup(IFolder):

    form.widget('confidentiality_activated', RadioFieldWidget)
    confidentiality_activated = schema.Bool(
        title=_(u'Activate the "confidential" option'),
        required=False,
        default=False,
    )

    form.widget('to_be_printed_activated', RadioFieldWidget)
    to_be_printed_activated = schema.Bool(
        title=_(u'Activate the "to be printed" option'),
        required=False,
        default=False,
    )

    form.widget('signed_activated', RadioFieldWidget)
    signed_activated = schema.Bool(
        title=_(u'Activate the "to_sign and signed" options'),
        required=False,
        default=False,
    )

    form.widget('publishable_activated', RadioFieldWidget)
    publishable_activated = schema.Bool(
        title=_(u'Activate the "publishable" option'),
        required=False,
        default=False,
    )


class CategoryGroup(Container):
    implements(ICategoryGroup)


class CategoryGroupSchemaPolicy(DexteritySchemaPolicy):

    def bases(self, schema_name, tree):
        return (ICategoryGroup, )



# -*- coding: utf-8 -*-

from zope.i18nmessageid import MessageFactory

# enable :json type converter
import imio.helpers.converters  # noqa
import logging


logger = logging.getLogger('collective.iconifiedcategory')

CAT_SEPARATOR = '_-_'
CSS_SEPARATOR = '-'

_ = MessageFactory('collective.iconifiedcategory')



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from collective.iconifiedcategory import _
from plone.namedfile.interfaces import IImageScaleTraversable
from zope import schema
from zope.component.interfaces import IObjectEvent
from zope.interface import Attribute
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.viewlet.interfaces import IViewletManager


class IBelowCategorizedElementsManager(IViewletManager):
    """A viewlet manager that sits just below categorized elements table."""


class ICollectiveIconifiedCategoryLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IIconifiedCategoryConfig(Interface):
    pass


class IIconifiedCategoryGroup(Interface):
    pass


class IIconifiedContent(Interface):
    pass


class IIconifiedInfos(Interface):
    pass


class IIconifiedPrintable(Interface):
    pass


class IIconifiedPreview(Interface):
    pass


class ICategorizedTable(Interface):
    pass


class ICategorizedPrint(ICategorizedTable):
    pass


class ICategorizedConfidential(ICategorizedTable):
    pass


class ICategorizedPublishable(ICategorizedTable):
    pass


class ICategorizedSigned(ICategorizedTable):
    pass


class IICImageScaleTraversable(IImageScaleTraversable):
    pass


class IIconifiedCategorySubtyper(Interface):

    have_categorized_elements = schema.Bool(
        u'Is current object contains categorized elements',
        readonly=True,
    )


class IIconifiedCategorySettings(Interface):

    sort_categorized_tab = schema.Bool(
        title=_(
            u'Sort categorized elements on categorized tab alphabetically '
            u'automatically. Uncheck to sort elements manually'),
        default=True,
    )

    categorized_childs_infos_columns_threshold = schema.Int(
        title=_(u'Maximum number of elements to display by columns when '
                u'displaying categorized elements of same category in the '
                u'tooltipster widget'),
        default=25,
    )

    filesizelimit = schema.Int(
        title=_(u'Filesize limit in bytes enabling a warning'),
        description=_(u'If the categorized element is a file, the user will '
                      u'get a warning whenever the filesize is bigger than '
                      u'defined value. <span style="color: red">Take care that '
                      u'if you change this value, you will have to update the '
                      u'categoized elements stored informations.</span> '),
        default=5000000,
    )


# Events

class ICategorizedElementsUpdatedEvent(IObjectEvent):
    pass


class IIconifiedAttrChangedEvent(IObjectEvent):

    attr_name = Attribute("The attribute name")
    old_values = Attribute("The old values")
    new_values = Attribute("The new values")


class IIconifiedModifiedEvent(IObjectEvent):
    pass


class IIconifiedCategoryChangedEvent(IObjectEvent):
    pass



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import collective.iconifiedcategory


class CollectiveIconifedCategoryLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        z2.installProduct(app, 'Products.DateRecurringIndex')
        self.loadZCML(package=collective.iconifiedcategory)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.iconifiedcategory:testing')


COLLECTIVE_ICONIFIED_CATEGORY_FIXTURE = CollectiveIconifedCategoryLayer()


COLLECTIVE_ICONIFIED_CATEGORY_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_ICONIFIED_CATEGORY_FIXTURE,),
    name='CollectiveIconifedCategoryLayer:IntegrationTesting'
)


COLLECTIVE_ICONIFIED_CATEGORY_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_ICONIFIED_CATEGORY_FIXTURE,),
    name='CollectiveIconifedCategoryLayer:FunctionalTesting'
)


COLLECTIVE_ICONIFIED_CATEGORY_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_ICONIFIED_CATEGORY_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectiveIconifedCategoryLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
from collective.iconifiedcategory.adapter import CategorizedObjectAdapter


class TestingCategorizedObjectAdapter(CategorizedObjectAdapter):

    def can_view(self):
        """Return False if element is confidential."""
        if self.categorized_obj.confidential:
            return False
        else:
            return True



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.eeafaceted.z3ctable.browser.views import ExtendedCSSTable
from collective.eeafaceted.z3ctable.columns import BaseColumn
from collective.iconifiedcategory import _
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.interfaces import ICategorizedConfidential
from collective.iconifiedcategory.interfaces import ICategorizedPrint
from collective.iconifiedcategory.interfaces import ICategorizedPublishable
from collective.iconifiedcategory.interfaces import ICategorizedSigned
from collective.iconifiedcategory.interfaces import ICategorizedTable
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from plone import api
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import _checkPermission
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implements

import html


class CategorizedTabView(BrowserView):

    def table_render(self, portal_type=None):
        self.portal_type = portal_type
        table = self.context.restrictedTraverse('@@iconifiedcategory_table')
        self.table = table
        self.table.portal_type = portal_type
        self._prepare_table_render()
        self.table.update()
        return self.table.render()

    def _prepare_table_render(self, **kwargs):
        if self.show('confidentiality'):
            alsoProvides(self.table, ICategorizedConfidential)
        if self.show('to_be_printed'):
            alsoProvides(self.table, ICategorizedPrint)
        if self.show('publishable'):
            alsoProvides(self.table, ICategorizedPublishable)
        if self.show('signed'):
            alsoProvides(self.table, ICategorizedSigned)

    def _config(self):
        """ """
        return utils.get_config_root(self.context)

    def show(self, action_type):
        """ """
        config = self._config()
        if config.portal_type == 'ContentCategoryConfiguration':
            return True
        attr_config = '{0}_activated'.format(action_type)
        show = getattr(config, attr_config) and self._show_column(action_type)
        return show

    def _show_column(self, action_type):
        """Made to be overrided."""
        return True


class CategorizedContent(object):

    def __init__(self, context, content):
        self.context = context
        self.UID = content['UID']
        self._metadata = context.categorized_elements.get(content['UID'], {})

    def __getattr__(self, key):
        if key in self._metadata:
            return self._metadata.get(key)
        return getattr(self.context, key)

    def getObject(self):
        obj = getattr(self, 'obj', None)
        if not obj:
            obj = self.context.get(self._metadata['id'])
            self.obj = obj
        return obj

    # make call to _unrestrictedGetObject behaves like getObject
    _unrestrictedGetObject = getObject

    @property
    def Description(self):
        return self.getObject().Description()

    @property
    def Creator(self):
        return self.getObject().Creator()

    @property
    def CreationDate(self):
        return self.created

    @property
    def created(self):
        return self.getObject().created()

    @property
    def ModificationDate(self):
        return self.modified

    @property
    def modified(self):
        return self.getObject().modified()

    def getPath(self):
        portal_path = '/'.join(api.portal.get().getPhysicalPath())
        path = '{0}/{1}'.format(portal_path, self.relative_url)
        return path

    def getURL(self, suffix=None):
        """ """
        portal_url = api.portal.get().absolute_url()
        suffix = suffix or self.relative_url
        return portal_url + '/' + suffix


class CategorizedTable(ExtendedCSSTable, BrowserView):
    implements(ICategorizedTable)

    cssClasses = {'table': 'listing iconified-listing nosort'}

    cssClassEven = u'odd'
    cssClassOdd = u'even'
    # do not sort, keep order of position in parent
    sortOn = None
    batchSize = 999
    startBatchingAt = 999

    def __init__(self, context, request, portal_type=None):
        """If received, this let's filter table for a given portal_type."""
        self.portal_type = portal_type
        super(CategorizedTable, self).__init__(context, request)
        self.portal = api.portal.get()

    @property
    def values(self):
        if not getattr(self, '_v_stored_values', []):
            sort_on = 'getObjPositionInParent'
            sort_categorized_tab = api.portal.get_registry_record(
                'sort_categorized_tab',
                interface=IIconifiedCategorySettings,
            )
            if sort_categorized_tab is True:
                sort_on = None
            data = [
                CategorizedContent(self.context, content) for content in
                utils.get_categorized_elements(
                    self.context,
                    result_type='dict',
                    portal_type=self.portal_type,
                    sort_on=sort_on,
                )
            ]
            self._v_stored_values = data
        return self._v_stored_values

    def render(self):
        if not len(self.rows):
            return _(
                'no_element_to_display',
                default="<span class='discreet iconified_category_no_element'>No element to display.</span>")
        return super(CategorizedTable, self).render()


class TitleColumn(BaseColumn):
    header = _(u'Title')
    weight = 20
    attrName = 'title'
    # we manage escape here manually
    escape = False

    def renderCell(self, content):
        pattern = (
            u'<a href="{link}" alt="{title}" title="{title}" target="{target}">'
            u'<img src="{icon}" alt="{category}" title="{category}" />'
            u' {title}</a><p class="discreet">{description}</p>'
        )
        url = content.getURL()
        target = ''
        if content.preview_status == 'converted':
            url = u'{0}/documentviewer#document/p1'.format(url)
            target = '_blank'
        return pattern.format(
            link=url,
            title=html.escape(getattr(content, self.attrName).decode('utf-8')),
            target=target,
            icon=content.icon_url,
            category=html.escape(safe_unicode(content.category_title)),
            description=html.escape(safe_unicode(content.Description)),
        )


class CategoryColumn(BaseColumn):
    header = _(u'Category')
    weight = 30
    attrName = 'category_title'
    escape = False

    def renderCell(self, content):
        category_title = safe_unicode(content.category_title)
        if content.subcategory_title:
            category_title = u"{0} / {1}".format(category_title,
                                                 safe_unicode(content.subcategory_title))
        return html.escape(category_title)


class AuthorColumn(BaseColumn):
    header = _(u'Author')
    weight = 40

    def renderCell(self, content):
        return html.escape(content.Creator)


class CreationDateColumn(BaseColumn):
    header = _(u'Creation date')
    weight = 50

    def renderCell(self, content):
        return api.portal.get_localized_time(
            datetime=content.created,
            long_format=True,
        )


class LastModificationColumn(BaseColumn):
    header = _(u'Last modification')
    weight = 60

    def renderCell(self, content):
        if content.created == content.modified:
            return ''
        return api.portal.get_localized_time(
            datetime=content.modified,
            long_format=True,
        )


class FilesizeColumn(BaseColumn):
    header = _(u'Filesize')
    weight = 70
    escape = False

    def renderHeadCell(self):
        """ """
        header = super(FilesizeColumn, self).renderHeadCell()
        total = sum([v.filesize for v in self.table.values
                     if v.filesize is not None])
        header += u"<p>(Total: {0})</p>".format(utils.render_filesize(total))
        return header

    def renderCell(self, content):
        if getattr(content, 'filesize', None) is None:
            return ''
        return utils.render_filesize(content.filesize)


class IconClickableColumn(BaseColumn):
    action_view = ''
    escape = False

    def _deactivated_is_useable(self):
        '''Is deactivated value a useable one?'''
        return False

    def get_url(self, content):
        if not self._deactivated_is_useable() and self.is_deactivated(content):
            return '#'
        return '{url}/@@{action}'.format(
            url=content.getURL(),
            action=self.get_action_view(content),
        )

    def get_action_view(self, content):
        return self.action_view

    def alt(self, content):
        return self.header

    def is_deactivated(self, content):
        return getattr(content, self.attrName, False) is None

    def is_editable(self, content):
        view = getMultiAdapter((content.getObject(), self.request), name=self.action_view)
        return view._may_set_values({})

    def is_active(self, content):
        return getattr(content, self.attrName, False)

    def css_class(self, content):
        is_deactivated = self.is_deactivated(content)
        if not self._deactivated_is_useable() and is_deactivated:
            return ' deactivated'
        base_css = self.is_active(content) and ' active' or ''
        if is_deactivated:
            base_css = ' deactivated' + base_css
        if self.is_editable(content):
            return '{0} editable'.format(base_css)
        return base_css

    def renderCell(self, content):
        link = (u'<a href="{0}" class="iconified-action{1}" alt="{2}" '
                u'title="{2}"></a>')
        return link.format(
            self.get_url(content),
            self.css_class(content),
            self.alt(content),
        )


class PrintColumn(IconClickableColumn):
    header = _(u'To be printed')
    cssClasses = {'td': 'iconified-print'}
    weight = 80
    attrName = 'to_print'
    action_view = 'iconified-print'

    def alt(self, content):
        return translate(
            utils.print_message(content),
            domain='collective.iconifiedcategory',
            context=self.table.request,
        )


class ConfidentialColumn(IconClickableColumn):
    header = _(u'Confidential')
    cssClasses = {'td': 'iconified-confidential'}
    weight = 90
    attrName = 'confidential'
    action_view = 'iconified-confidential'

    def alt(self, content):
        return translate(
            utils.boolean_message(content, attr_name='confidential'),
            domain='collective.iconifiedcategory',
            context=self.table.request,
        )


class SignedColumn(IconClickableColumn):
    header = _(u'Signed')
    cssClasses = {'td': 'iconified-signed'}
    weight = 95
    attrName = 'signed'
    action_view = 'iconified-signed'

    def alt(self, content):
        return translate(
            utils.signed_message(content),
            domain='collective.iconifiedcategory',
            context=self.table.request,
        )

    def _deactivated_is_useable(self):
        '''Is deactivated value a useable one?'''
        return True

    def is_deactivated(self, content):
        return not getattr(content, 'to_sign', True)


class PublishableColumn(IconClickableColumn):
    header = _(u'Publishable')
    cssClasses = {'td': 'iconified-publishable'}
    weight = 98
    attrName = 'publishable'
    action_view = 'iconified-publishable'

    def alt(self, content):
        return translate(
            utils.boolean_message(content, attr_name='publishable'),
            domain='collective.iconifiedcategory',
            context=self.table.request,
        )


class ActionColumn(BaseColumn):
    header = u''
    weight = 100
    escape = False

    def renderCell(self, content):
        link = u'<a href="{href}"><img src="{src}" title="{title}" /></a>'
        render = []
        if _checkPermission(ModifyPortalContent, content):
            render.append(link.format(
                href=u'{0}/edit'.format(content.getURL()),
                src=u'{0}/edit.gif'.format(content.getURL()),
                title=_('Edit'),
            ))
        if content.download_url:
            render.append(link.format(
                href=content.download_url,
                src=u'{0}/download_icon.png'.format(content.getURL()),
                title=_('Download'),
            ))
        if content.preview_status == 'converted':
            render.append(link.format(
                href=u'{0}/documentviewer#document/p1'.format(content.getURL()),
                src=u'{0}/file_icon.png'.format(content.getURL()),
                title=_('Preview'),
            ))
        return u''.join(render)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.documentviewer.async import queueJob
from collective.iconifiedcategory import _
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.content.category import ICategory
from collective.iconifiedcategory.content.subcategory import ISubcategory
from collective.iconifiedcategory.event import IconifiedAttrChangedEvent
from collective.iconifiedcategory.interfaces import IIconifiedPrintable
from imio.helpers.cache import invalidate_cachekey_volatile_for
from plone import api
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import Redirect
from zope.component import getAdapter
from zope.event import notify


def categorized_content_created(obj, event):

    if hasattr(obj, 'content_category'):
        # if 'to_print' and 'confidential' are managed manually,
        # we may defer events if relevant value found in the REQUEST
        if obj.REQUEST.get('defer_categorized_content_created_event', False):
            return

        # set default values for to_print, confidential and to_sign/signed
        try:
            category = utils.get_category_object(obj, obj.content_category)
        except KeyError:
            return
        # left False if to_print/confidential/to_sign
        # not enabled on ContentCategoryGroup
        category_group = category.get_category_group(category)

        # only set default value if obj was not created with a to_print=True
        if category_group.to_be_printed_activated and not getattr(obj, 'to_print', False):
            obj.to_print = category.to_print
            # notifying IconifiedAttrChangedEvent for 'to_print' is done in categorized_content_updated
        elif not category_group.to_be_printed_activated:
            obj.to_print = False

        # only set default value if obj was not created with a confidential=True
        if category_group.confidentiality_activated and not getattr(obj, 'confidential', False):
            obj.confidential = category.confidential
            notify(IconifiedAttrChangedEvent(
                obj,
                'confidential',
                old_values={},
                new_values={'confidential': obj.confidential},
                is_created=True
            ))
        elif not category_group.confidentiality_activated:
            obj.confidential = False

        # only set default value if obj was not created with a to_sign=True or signed=True
        if category_group.signed_activated and not (getattr(obj, 'to_sign', False) or getattr(obj, 'signed', False)):
            obj.to_sign = category.to_sign
            obj.signed = category.signed
            notify(IconifiedAttrChangedEvent(
                obj,
                'to_sign',
                old_values={},
                new_values={'to_sign': obj.to_sign,
                            'signed': obj.signed},
                is_created=True
            ))

        elif not category_group.signed_activated:
            obj.to_sign = False
            obj.signed = False

        # only set default value if obj was not created with a publishable=True
        if category_group.publishable_activated and not getattr(obj, 'publishable', False):
            obj.publishable = category.publishable
            notify(IconifiedAttrChangedEvent(
                obj,
                'publishable',
                old_values={},
                new_values={'publishable': obj.publishable},
                is_created=True
            ))
        elif not category_group.publishable_activated:
            obj.publishable = False

        # to_print changed event is managed in categorized_content_updated
        categorized_content_updated(event, is_created=True)

        if utils.is_file_type(obj.portal_type):
            file_field_name = IPrimaryFieldInfo(obj).fieldname
            size = getattr(obj, file_field_name).size
            if utils.warn_filesize(size):
                plone_utils = api.portal.get_tool('plone_utils')
                plone_utils.addPortalMessage(
                    _("The annex that you just added has a large size and "
                      "could be difficult to download by users wanting to "
                      "view it!"), type='warning')


def content_updated(obj, event):
    if hasattr(obj, 'content_category'):
        categorized_content_updated(event)


def categorized_content_updated(event, is_created=False):
    obj = event.object

    if hasattr(obj, 'content_category'):
        category = utils.get_category_object(obj, obj.content_category)

        if category.show_preview in (1, 2):
            queueJob(obj)

        if hasattr(obj, 'to_print'):
            # if current 'to_print' is None, it means that current content
            # could not be printable, but as it changed,
            # in this case we use the default value
            if obj.to_print is None:
                category_group = category.get_category_group(category)
                if category_group.to_be_printed_activated:
                    obj.to_print = category.to_print

            adapter = getAdapter(obj, IIconifiedPrintable)
            adapter.update_object()
            notify(IconifiedAttrChangedEvent(
                obj,
                'to_print',
                old_values={'to_print': obj.to_print},
                new_values={'to_print': obj.to_print},
                is_created=is_created
            ))
        # we may defer call to utils.update_categorized_elements
        # if relevant value found in the REQUEST
        # this is useful when adding several categorized elements without
        # calling update_categorized_elements between every added element
        if obj.REQUEST.get('defer_update_categorized_elements', False):
            return

        utils.update_categorized_elements(obj.aq_parent, obj, category)


def content_category_updated(event):
    if hasattr(event.object, 'content_category'):
        obj = event.object
        target = utils.get_category_object(obj, obj.content_category)
        utils.update_categorized_elements(
            obj.aq_parent,
            obj,
            target,
            limited=False,
            sort=event.sort,
            logging=True
        )


def categorized_content_removed(event):
    if hasattr(event.object, 'content_category'):
        obj = event.object
        utils.remove_categorized_element(obj.aq_parent, obj)


def categorized_content_container_cloned(container, event):
    if container.REQUEST.get('defer_update_categorized_elements', False):
        return
    utils.update_all_categorized_elements(container)


def category_before_remove(obj, event):
    # do not fail if removing the Plone Site
    if not event.object.meta_type == 'Plone Site' and \
       ICategory.providedBy(obj) is True:
        if utils.has_relations(obj) is True:
            IStatusMessage(obj.REQUEST).addStatusMessage(
                _('This category or one of is subcategory are used by '
                  'another object and cannot be deleted'),
                type='error',
            )
            raise Redirect(obj.REQUEST.get('HTTP_REFERER'))
        _cookCssResources()


def subcategory_before_remove(obj, event):
    # do not fail if removing the Plone Site
    if not event.object.meta_type == 'Plone Site' and \
       ISubcategory.providedBy(obj) is True:
        if utils.has_relations(obj) is True:
            IStatusMessage(obj.REQUEST).addStatusMessage(
                _('This subcategory is used by another object and cannot be '
                  'deleted'),
                type='error',
            )
            raise Redirect(obj.REQUEST.get('HTTP_REFERER'))


def category_moved(obj, event):
    if event.oldParent is None or event.newParent is None:
        return
    if utils.has_relations(obj) is True:
        IStatusMessage(obj.REQUEST).addStatusMessage(
            _('This category or one of is subcategory are used by '
              'another object and cannot be deleted'),
            type='error',
        )
        raise Redirect(obj.REQUEST.get('HTTP_REFERER'))
    _cookCssResources()


def subcategory_moved(obj, event):
    if event.oldParent is None or event.newParent is None:
        return
    if utils.has_relations(obj) is True:
        IStatusMessage(obj.REQUEST).addStatusMessage(
            _('This subcategory is used by another object and cannot be '
              'deleted'),
            type='error',
        )
        raise Redirect(obj.REQUEST.get('HTTP_REFERER'))


def _cookCssResources():
    # recook portal_css because we need
    # iconified-category.css to be compiled again as it is cached
    portal_css = api.portal.get_tool('portal_css')
    portal_css.cookResources()


def category_created(category, event):
    # make sure the 'listing' scale image is created
    category.restrictedTraverse('@@images').scale(scale='listing')
    _cookCssResources()


def container_modified(obj, event):
    """When a category container (so a CategoryGroup or a Category)
       is modified (meaning element added/removed/position changed)
       invalidate date used for utils.get_ordered_categories caching."""
    invalidate_cachekey_volatile_for(
        'collective.iconifiedcategory.utils.get_ordered_categories', get_again=True)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.iconifiedcategory.testing import COLLECTIVE_ICONIFIED_CATEGORY_INTEGRATION_TESTING
from plone import api

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.iconifiedcategory is properly installed."""

    layer = COLLECTIVE_ICONIFIED_CATEGORY_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.iconifiedcategory is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'collective.iconifiedcategory'))

    def test_browserlayer(self):
        """Test that ICollectiveIconifiedCategoryLayer is registered."""
        from collective.iconifiedcategory.interfaces import ICollectiveIconifiedCategoryLayer
        from plone.browserlayer import utils
        self.assertIn(ICollectiveIconifiedCategoryLayer,
                      utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_ICONIFIED_CATEGORY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.iconifiedcategory'])

    def test_product_uninstalled(self):
        """Test if collective.iconifiedcategory is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'collective.iconifiedcategory'))



# -*- coding: utf-8 -*-

from collective.iconifiedcategory import utils
from collective.iconifiedcategory.content.base import ICategorize
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer import indexer
from Products.PluginIndexes.common.UnIndex import _marker


@indexer(ICategorize)
def enabled(obj):
    """
    Indexes the 'sortable_title 'enabled' attribute.
    """
    return obj.enabled


@indexer(IDexterityContent)
def content_category_uid(obj):
    """Index the category_uid"""
    if not hasattr(obj, 'content_category'):
        return
    try:
        category_object = utils.get_category_object(obj, obj.content_category)
    except KeyError:
        return _marker
    return category_object.UID()



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory.interfaces import ICategorizedElementsUpdatedEvent
from collective.iconifiedcategory.interfaces import IIconifiedAttrChangedEvent
from collective.iconifiedcategory.interfaces import IIconifiedCategoryChangedEvent
from collective.iconifiedcategory.interfaces import IIconifiedModifiedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class IconifiedModifiedEvent(ObjectEvent):
    implements(IIconifiedModifiedEvent)


class IconifiedCategoryChangedEvent(ObjectEvent):
    implements(IIconifiedCategoryChangedEvent)

    def __init__(self, object, category, sort=False):
        super(IconifiedCategoryChangedEvent, self).__init__(object)
        self.category = category
        self.sort = sort


class IconifiedAttrChangedEvent(ObjectEvent):
    implements(IIconifiedAttrChangedEvent)

    def __init__(self, object, attr_name, old_values, new_values, is_created=False):
        super(IconifiedAttrChangedEvent, self).__init__(object)
        self.attr_name = attr_name
        self.old_values = old_values
        self.new_values = new_values
        self.is_created = is_created


class CategorizedElementsUpdatedEvent(ObjectEvent):
    implements(ICategorizedElementsUpdatedEvent)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from Acquisition import aq_base
from collective.documentviewer.settings import GlobalSettings
from collective.documentviewer.settings import Settings
from collective.documentviewer.utils import allowedDocumentType
from collective.iconifiedcategory import utils
from collective.iconifiedcategory.content.subcategory import ISubcategory
from collective.iconifiedcategory.interfaces import IIconifiedPreview
from collective.iconifiedcategory.utils import _modified
from plone import api
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import IImage
from plone.app.contenttypes.interfaces import ILink
from plone.indexer.interfaces import IIndexableObject
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import _checkPermission
from zope.annotation import IAnnotations
from zope.component import queryMultiAdapter


class CategorizedObjectInfoAdapter(object):

    def __init__(self, context):
        self.obj = aq_base(context)
        self.context = context

    def _get_basic_infos(self, category):
        """Return the basic informations for the object"""
        infos = {
            'category_uid': category.category_uid,
            'category_id': category.category_id,
            'category_title': category.category_title,
            'subcategory_uid': None,
            'subcategory_id': None,
            'subcategory_title': None,
            # this will be useable for caching for example
            'last_updated': self._last_updated,
            'icon_url': utils.get_category_icon_url(category),
            'to_be_printed_activated': self._to_be_printed_activated(category),
            'confidentiality_activated': self._confidentiality_activated(category),
            'signed_activated': self._signed_activated(category),
            'publishable_activated': self._publishable_activated(category),
            'to_print': self._to_print,
            'confidential': self._confidential,
            'to_sign': self._to_sign,
            'signed': self._signed,
            'publishable': self._publishable,
            'show_preview': self._show_preview(category),
        }
        # update subcategory infos if any
        if ISubcategory.providedBy(category):
            infos['subcategory_uid'] = category.UID()
            infos['subcategory_id'] = category.getId()
            infos['subcategory_title'] = category.Title()
        return infos

    def get_infos(self, category, limited=False):
        filesize = self._filesize
        portal_url = api.portal.get_tool('portal_url')
        base_infos = self._get_basic_infos(category)
        if limited is True:
            return base_infos
        infos = {
            'title': self.obj.Title(),
            'description': self.obj.Description(),
            'id': self.obj.getId(),
            'relative_url': portal_url.getRelativeUrl(self.context),
            'download_url': self._download_url,
            'portal_type': self.obj.portal_type,
            'filesize': filesize,
            'warn_filesize': utils.warn_filesize(filesize),
            'contentType': self._file_contentType,
            'preview_status': self._preview_status,
            'allowedRolesAndUsers': self._allowedRolesAndUsers,
        }
        infos.update(base_infos)
        return infos

    @property
    def _category(self):
        """Return the category instead of the subcategory"""
        return '_-_'.join(self.obj.content_category.split('_-_')[:3])

    @property
    def _download_url(self):
        """Return the download url (None by default) for the current object"""
        url = u'{url}/@@download'
        portal_url = api.portal.get_tool('portal_url')
        if IFile.providedBy(self.obj) or IImage.providedBy(self.obj):
            return url.format(
                url=portal_url.getRelativeUrl(self.context))

    @property
    def _filesize(self):
        """Return the filesize if the contenttype is a File or an Image"""
        if IFile.providedBy(self.obj):
            return self.obj.file.size
        if IImage.providedBy(self.obj):
            return self.obj.image.size

    @property
    def _file_contentType(self):
        """Return the contentType if it is a File or an Image"""
        if IFile.providedBy(self.obj):
            return self.obj.file.contentType
        if IImage.providedBy(self.obj):
            return self.obj.image.contentType

    @property
    def _last_updated(self):
        """When updating some attributes directly or annotation,
           "modified" is not changed."""
        return _modified(self.context)

    def _to_be_printed_activated(self, category):
        category_group = category.get_category_group()
        return category_group.to_be_printed_activated

    @property
    def _to_print(self):
        return getattr(self.obj, 'to_print', False)

    def _confidentiality_activated(self, category):
        category_group = category.get_category_group()
        return category_group.confidentiality_activated

    @property
    def _confidential(self):
        return getattr(self.obj, 'confidential', False)

    def _signed_activated(self, category):
        category_group = category.get_category_group()
        return category_group.signed_activated

    @property
    def _to_sign(self):
        return getattr(self.obj, 'to_sign', False)

    @property
    def _signed(self):
        return getattr(self.obj, 'signed', False)

    def _publishable_activated(self, category):
        category_group = category.get_category_group()
        return category_group.publishable_activated

    @property
    def _publishable(self):
        return getattr(self.obj, 'publishable', False)

    def _show_preview(self, category):
        return category.show_preview

    @property
    def _preview_status(self):
        return IIconifiedPreview(self.obj).status

    @property
    def _allowedRolesAndUsers(self):
        catalog = api.portal.get_tool('portal_catalog')
        # use self.context that is acquisition wrapped, need this to get every local_roles
        wrapper = queryMultiAdapter((self.context, catalog, ), IIndexableObject)
        return wrapper.allowedRolesAndUsers


class CategorizedObjectPrintableAdapter(object):

    def __init__(self, context):
        self.context = context

    @property
    def is_printable(self):
        if ILink.providedBy(self.context):
            return False
        if IFile.providedBy(self.context):
            return IIconifiedPreview(self.context).is_convertible()
        if IImage.providedBy(self.context):
            return True
        return True

    @property
    def error_message(self):
        return u'Can not be printed'

    def update_object(self):
        self.context.to_print_message = None
        if self.is_printable is False:
            # None means 'deactivated'
            self.context.to_print = None
            self.context.to_print_message = self.error_message


class CategorizedObjectAdapter(object):

    def __init__(self, context, request, categorized_obj):
        self.context = context
        self.request = request
        self.categorized_obj = categorized_obj

    def can_view(self):
        """By default, check that current user may View the context.
           Indeed for some advanced management (@@download), views are
           declared permission="zope2.Public"."""
        if _checkPermission(View, self.context):
            return True


class CategorizedObjectPreviewAdapter(object):
    """Base adapter to verify the preview conversion status"""

    def __init__(self, context):
        self.context = context

    @property
    def status(self):
        """
          Returns the conversion status of context.
        """
        # not_convertable or awaiting conversion?
        if not self.is_convertible():
            return 'not_convertable'

        # under conversion?
        ann = IAnnotations(self.context)['collective.documentviewer']
        if 'successfully_converted' not in ann:
            if 'filehash' in ann or ann.get('converting', False) is True:
                return 'in_progress'
            return 'not_converted'

        if not ann['successfully_converted'] is True:
            return 'conversion_error'

        return 'converted'

    @property
    def converted(self):
        """ """
        return self.status == 'converted'

    def is_convertible(self):
        """
          Check if the context is convertible (hopefully).
        """
        # collective.documentviewer add an entry to the annotations
        annotations = IAnnotations(self.context)
        if 'collective.documentviewer' not in annotations.keys():
            Settings(self.context)

        settings = GlobalSettings(api.portal.get())
        return allowedDocumentType(self.context,
                                   settings.auto_layout_file_types)


class IconifiedCategoryGroupAdapter(object):

    def __init__(self, config, context):
        self.config = config
        self.context = context

    def get_group(self):
        return self.config

    def get_every_categories(self, only_enabled=True):
        return utils.get_categories(self.context, the_objects=True, only_enabled=only_enabled)



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone.app.layout.viewlets import common as base


class CategorizedChildViewlet(base.ViewletBase):
    """ """



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import utils
from collective.iconifiedcategory.content.category import ICategory
from collective.iconifiedcategory.event import CategorizedElementsUpdatedEvent
from collective.iconifiedcategory.event import IconifiedCategoryChangedEvent
from plone import api
from Products.Five import BrowserView
from zope.event import notify
from zope.i18n import translate


class UpdateCategorizedElementsBase(BrowserView):

    def __init__(self, context, request):
        super(UpdateCategorizedElementsBase, self).__init__(context, request)
        self._notified = []
        self.sort = self.request.get(
            'sort_updated_categorized_elements', False) is True or False

    def _notify(self, brain):
        if brain.UID in self._notified:
            return
        self._notified.append(brain.UID)
        event = IconifiedCategoryChangedEvent(brain.getObject(), self.context, sort=self.sort)
        notify(event)

    def notify_category_updated(self, obj):
        brains = [b for b in utils.get_back_references(obj)]
        if ICategory.providedBy(obj):
            for subcategory in obj.listFolderContents():
                brains.extend(utils.get_back_references(subcategory))
        for b in brains:
            self._notify(b)

    def _finished(self):
        msg = translate('Elements updated!',
                        domain='collective.iconifiedcategory',
                        context=self.request)
        notify(CategorizedElementsUpdatedEvent(self.context))
        api.portal.show_message(msg, request=self.request)
        self.request.RESPONSE.redirect(self.context.absolute_url())


class UpdateCategorizedElementsConfig(UpdateCategorizedElementsBase):

    def index(self):
        brains = api.content.find(
            context=self.context,
            content_type='ContentCategory',
        )
        for b in brains:
            self.notify_category_updated(b.getObject())
        self._finished()


class UpdateCategorizedElementsCategory(UpdateCategorizedElementsBase):

    def index(self):
        self.notify_category_updated(self.context)
        self._finished()



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory.tests.base import BaseTestCase


class TestIconifiedCategorySubtyper(BaseTestCase):

    def _subtyper(self, obj, prop):
        subtyper = obj.restrictedTraverse('@@iconified_subtyper')
        return getattr(subtyper, prop)

    def test_have_categorized_elements(self):
        """Test the have_categorized_elements property"""
        self.assertTrue(self._subtyper(
            self.portal,
            'have_categorized_elements',
        ))
        self.assertFalse(self._subtyper(
            self.portal['config'],
            'have_categorized_elements',
        ))

    def test_on_config(self):
        """Test the on_config property"""
        self.assertFalse(self._subtyper(self.portal, 'on_config'))
        self.assertTrue(self._subtyper(self.portal['config'], 'on_config'))
        self.assertTrue(self._subtyper(
            self.portal['config']['group-1'],
            'on_config',
        ))
        self.assertTrue(self._subtyper(
            self.portal['config']['group-1']['category-1-1'],
            'on_config',
        ))
        self.assertTrue(self._subtyper(
            self.portal['config']['group-1']['category-1-1']['subcategory-1-1-1'],
            'on_config',
        ))



# -*- coding: utf-8 -*-
"""
collective.iconifiedcategory
----------------------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _
from collective.iconifiedcategory import utils
from imio.helpers.content import find
from zope.schema.vocabulary import SimpleVocabulary

import html


class CategoryVocabulary(object):

    def _get_categories(self, context, only_enabled=True):
        """Return categories to display in the vocabulary.
           This needs to return a list of category objects."""
        categories = utils.get_categories(context, the_objects=True, only_enabled=only_enabled)
        return categories

    def _get_subcategories(self, context, category, only_enabled=True):
        """Return subcategories for given category.
           This needs to return a list of subcategory brains."""
        query = {'object_provides': 'collective.iconifiedcategory.content.subcategory.ISubcategory'}
        if only_enabled:
            query['enabled'] = True
        subcategories = find(
            context=category,
            unrestricted=True,
            **query
        )
        return subcategories

    def __call__(self, context, use_category_uid_as_token=False, only_enabled=True):
        terms = []
        categories = self._get_categories(context, only_enabled=only_enabled)
        for category in categories:
            if use_category_uid_as_token:
                category_id = category.UID()
            else:
                category_id = utils.calculate_category_id(category)
            category_title = html.escape(category.Title())
            if category.only_pdf:
                category_title = category_title + ' [PDF!]'
            if category.show_preview != 0:
                category_title = category_title + ' [Preview!]'
            terms.append(SimpleVocabulary.createTerm(
                category_id,
                category_id,
                category_title,
            ))
            subcategories = self._get_subcategories(context, category, only_enabled=only_enabled)
            for subcategory in subcategories:
                subcategory = subcategory.getObject()
                if use_category_uid_as_token:
                    subcategory_id = subcategory.UID()
                else:
                    subcategory_id = utils.calculate_category_id(subcategory)
                subcategory_title = html.escape(subcategory.Title())
                if subcategory.only_pdf:
                    subcategory_title = subcategory_title + ' [PDF!]'
                if subcategory.show_preview != 0:
                    subcategory_title = subcategory_title + ' [Preview!]'
                terms.append(SimpleVocabulary.createTerm(
                    subcategory_id,
                    subcategory_id,
                    subcategory_title,
                ))
        return SimpleVocabulary(terms)


class EveryCategoryVocabulary(CategoryVocabulary):

    def __call__(self, context, use_category_uid_as_token=False, only_enabled=False):
        return super(EveryCategoryVocabulary, self).__call__(
            context,
            use_category_uid_as_token=use_category_uid_as_token,
            only_enabled=only_enabled)


class CategoryTitleVocabulary(CategoryVocabulary):

    def __call__(self, context, only_enabled=True):
        terms = []
        categories = self._get_categories(context, only_enabled=only_enabled)
        for category in categories:
            category_id = utils.calculate_category_id(category)
            if category.predefined_title:
                terms.append(SimpleVocabulary.createTerm(
                    category_id,
                    category_id,
                    category.predefined_title,
                ))
            subcategories = self._get_subcategories(context, category, only_enabled=only_enabled)
            for subcategory in subcategories:
                subcategory = subcategory.getObject()
                subcategory_id = utils.calculate_category_id(subcategory)
                if subcategory.predefined_title:
                    terms.append(SimpleVocabulary.createTerm(
                        subcategory_id,
                        subcategory_id,
                        subcategory.predefined_title,
                    ))
        return SimpleVocabulary(terms)


class EveryCategoryTitleVocabulary(CategoryTitleVocabulary):

    def __call__(self, context, only_enabled=False):
        return super(EveryCategoryTitleVocabulary, self).__call__(
            context,
            only_enabled=only_enabled)


class ShowPreviewVocabulary(object):

    def __call__(self, context):
        voc_terms = [
            SimpleVocabulary.createTerm(0, 0, _('No')),
            SimpleVocabulary.createTerm(1, 1, _('Yes')),
            SimpleVocabulary.createTerm(2, 2, _('Yes and hide download icon'))]

        return SimpleVocabulary(voc_terms)



# -*- coding: utf-8 -*-

from collective.iconifiedcategory import testing
from collective.iconifiedcategory.utils import _modified
from plone import api
from plone import namedfile
from plone.app.testing import login

import os
import unittest


class BaseTestCase(unittest.TestCase):

    layer = testing.COLLECTIVE_ICONIFIED_CATEGORY_FUNCTIONAL_TESTING

    @property
    def image(self):
        current_path = os.path.dirname(__file__)
        f = open(os.path.join(current_path, 'icône1.png'), 'r')
        return namedfile.NamedBlobFile(f.read(), filename=u'icône1.png')

    @property
    def file(self):
        current_path = os.path.dirname(__file__)
        f = open(os.path.join(current_path, 'file.txt'), 'r')
        return namedfile.NamedBlobFile(f.read(), filename=u'file.txt')

    @property
    def file_pdf(self):
        current_path = os.path.dirname(__file__)
        f = open(os.path.join(current_path, 'file.pdf'), 'r')
        return namedfile.NamedBlobFile(f.read(), filename=u'file.pdf')

    @property
    def icon(self):
        current_path = os.path.dirname(__file__)
        f = open(os.path.join(current_path, 'icône1.png'), 'r')
        return namedfile.NamedBlobFile(f.read(), filename=u'icône1.png')

    def _modified(self, obj):
        return _modified(obj)

    def setUp(self):
        self.maxDiff = None
        self.portal = self.layer['portal']
        self.config = self.portal['config']
        api.user.create(
            email='test@test.com',
            username='adminuser',
            password='secret',
        )
        api.user.grant_roles(
            username='adminuser',
            roles=['Manager'],
        )
        login(self.portal, 'adminuser')
        api.content.create(
            id='file_txt',
            type='File',
            file=self.file,
            container=self.portal,
            description='File description',
            content_category='config_-_group-1_-_category-1-1',
            to_print=False,
            confidential=False,
            publishable=False,
        )
        cat_id = 'config_-_group-1_-_category-1-1_-_subcategory-1-1-1'
        api.content.create(
            id='image',
            type='Image',
            image=self.image,
            container=self.portal,
            description='Image description',
            content_category=cat_id,
            to_print=False,
            confidential=False,
            publishable=False,
        )

    def tearDown(self):
        elements = ('file_txt', 'image')
        for element in elements:
            if element in self.portal:
                api.content.delete(self.portal[element])



# -*- coding: utf-8 -*-

from collective.iconifiedcategory.tests.base import BaseTestCase
from plone import api


class TestIconifiedCategoryCSS(BaseTestCase):

    def test__call__(self):
        view = self.portal.restrictedTraverse('@@collective-iconifiedcategory.css')
        css = view()
        self.assertTrue(".plone-config-group-1-category-1-1 " in css)
        self.assertTrue(u"background: transparent url("
                        u"'http://nohost/plone/config/group-1/category-1-1/@@download')"
                        in css)
        self.assertTrue(".plone-config-group-2-category-2-2 " in css)
        self.assertTrue(u"background: transparent url("
                        u"'http://nohost/plone/config/group-2/category-2-2/@@download')"
                        in css)
        self.assertTrue(".plone-config-group-2-category-2-3 " in css)
        self.assertTrue(u"background: transparent url("
                        u"'http://nohost/plone/config/group-2/category-2-3/@@download')"
                        in css)

        # delete the config
        api.content.delete(self.portal['file_txt'])
        api.content.delete(self.portal['image'])
        api.content.delete(self.portal['config'])
        self.assertEqual(view(), '')

    def test_css_recooked(self):
        """portal_css is recooked when a category is added/moved/removed."""
        def _current_css_cachekey():
            cachekey = [k for k, v in self.portal.portal_css.concatenatedResourcesByTheme['Plone Default'].items()
                        if 'collective-iconifiedcategory.css' in v and '-cachekey-' in k][0]
            return cachekey
        # portal_css is cooked, the collective-iconfiedcategory.css is stored in a cachekey cooked css
        cachekey1 = _current_css_cachekey()
        # add a category, css resources are cooked again
        category = api.content.create(
            type='ContentCategory',
            title='Brand new category',
            icon=self.icon,
            container=self.portal.config['group-1'],
        )
        cachekey2 = _current_css_cachekey()
        self.assertNotEqual(cachekey1, cachekey2)

        # rename the category so it is moved, css resources are cooked again
        category_parent = category.aq_inner.aq_parent
        category_parent.manage_renameObject(category.getId(), 'renamed_id')
        cachekey3 = _current_css_cachekey()
        self.assertNotEqual(cachekey2, cachekey3)

        # remove the category, css resources are cooked again
        api.content.delete(category)
        cachekey4 = _current_css_cachekey()
        self.assertNotEqual(cachekey3, cachekey4)
